 /*
** csp.c							(Lin  Zhong)
**								(1995 - '96)
**  		The call-sequence of functions
**		--------------------------------
**
**	At first, the excution-sequence of below functions is required:
**
**	0. void set_null_elem(ElemId x)
**		The element 'x' is defined as the NULL_Elem
**
**	1. void constant_num(int number)
**		 the number of constants
**
**	2. void constant(ElemId e)
**
**  # routines to define the ordered structure
**		
**	3. void order_num(int number)
**		the number of order-relations
**
**	4. void order_def_less(ElemId a, ElemId b)
**		es gelte a =< b
**
** 	5. check_unique() 	
**		check whether those least-upper-bound constraints in 
** 		the given order-relations are unique,
**		return: 0, unique; 
**		return: n, the number of new added constants.
**		return:-1, error.
**
**  # routines to define the constraints
**
**	6. void variable_num(int number)
**		the number of variables
**
**	7. void variable(CVarId x)
**
**	8. void binary_constraint_num(int number)
**		the number of binary_constraints
**
**	9. void constraint_def_ee(ElemId a, ElemId b):   a =< b 
**	   void constraint_def_ev(ElemId a, CVarId v):   a =< v 
**	   void constraint_def_ve(CVarId u, ElemId b):   u =< b 
**	   void constraint_def_vv(CVarId u, CVarId v):   u =< v 
**
**  # routines to define the upper-bound-constraint
**		v = a \lub b \lub u \lub v ...
**
**	10. void upper_bound_constraint_num(int number)
**		the number of  upper-bound-constraints
**
**	11. UbExpr ub_first_e(ElemId a)
**	    UbExpr ub_first_v(CVarId a)
**	    UbExpr ub_expr_e(UbExpr u, ElemId a):  return: u \lub a 
**	    UbExpr ub_expr_v(UbExpr u, CVarId v);  return: u \lub v 
**
**	    void constraint_def_eu(ElemId a, UbExpr u):   a = u 
**	    void constraint_def_vu(CVarId v, UbExpr u):   v = u 
**
**
**
**	12. solvable_first()
**		return:  1    satisfiable  with first solution, 
** 			      then you can call  sovlable_all() to get all 
** 			      solutions.
**          	return:  0    not satisfiable.
**		return:  -1   error.	
**
**
**    Then you can choose the below functions to excute:
**
**	 sovlable_all(): 	
**		return:  1    satisfiable  with all solutions
**          	return:  0    not satisfiable.
**		return:  -1   error.
**
**	 ElemId glb(ElemId a, ElemId b)
**		calculate the greatest-low-bound of element a and b,
** 		return: ElemId c, if c =a \lub b	
** 		return: NULL_Elem, if no c exist
**
**	ElemId lub(ElemId a, ElemId b)
**		calculate the least upper bound of element a and b,
** 		return: ElemId c, if c =a \lub b	
** 		return: NULL_Elem, if no c exist
**
**
**	int test_leq(ElemId x, ElemId y)
**		check whether there exist a order-relation x =< y in 
**		the given order-relations; return: 1, if yes; 
**		return: 0, if no; return:-1, error
**
**	int is_maximal( ElemId a)
**		return: 1, if a is the maximal element in 
**		the given order relation, i.e. if a<=x, then a=x.
** 		return: 0, if  not.
**
**	int is_minimal(ElemId a)
**		return: 1, if a is the minimal element in 
**		the given order relation, i.e. if x>=a , then x=a.
** 		return: 0, if not.
**
**	int less_neighbor(ElemId x)
**		To search the direct neighbor:y of x, 
**		with y<=x, x!=y, y is maximal.
** 		return: n, the number of neighbors of x;
** 		return: 0, no neighbor
**
**	void reset_constraints(void)
**		all constraints are cancelled, so that you can define new
** 		constraints.
**
**
**    Finally you must excute this function:
**
**	void free_space(void)
**		To release all the memory allocated
**
*/
	

#include "csp.h"
#include "checking.h"
#include <stdlib.h>


#define size1 200
#define size2 100
#define size3 3000
#define size4 50


struct node
        {
          ElemId head ;		/*head <= tail*/
	  ElemId tail ;
          struct node *head_link;  /* head_link: a pointer for head-link */
          struct node *tail_link;  /* tail_link: a pointer for tail-link */
        };

struct com_lab
	 {
	   int value;
	   int num;
	 };

struct two_values
	{
	   int value1;
	   int value2;
	};

ElemId NULL_Elem;
/* Definition of the NULL_Elem */

ElemId *domain;
/*domain[]: an array for the constants; */

ElemId *order1, *order2;
/* order1[], order2[]: arrays for the order-relations, 
** order1[i]<= order2[i], i=0,1...
*/

struct node *dom_node_tail=NULL;
/*dom_node_head[]: an array for the description of the order-relation*/

struct node *dom_node_head=NULL;
/*dom_node_head[]: an array for the description of the order-relation*/

struct node *order_node=NULL;
/*order_node[]: an array for the description of the order-relations */

struct node *new_order_node=NULL;

ElemId *newvalue=NULL;
/*newvalue[]: an array for the new added constants produced by check_unique()*/

ElemId *neighbor=NULL;
/*neighbor[]: an array for the neighbors with less_neighbor();
*/

CVarId *variables;
/*variables[]: an array for the variables*/

ElemId *ee1, *ee2, *ev1, *ev2, *ve1, *ve2, *vv1, *vv2;
/*
** ee1[],ee2[]: arrays for the Constraint-ee;order1[i]<= order2[i]:i=0,1... 
** ev1[],ev2[]: arrays for the constraint-ev;order1[i]<= order2[i]:i=0,1...
** ve1[],ve2[]: arrays for the constraint-ve;order1[i]<= order2[i]:i=0,1...
** vv1[],vv2[]: arrays for the constraint-vv.order1[i]<= order2[i]:i=0,1...
*/

struct node *var_node_head=NULL;
/*var_node_head[]: an array for the description of the constraints*/

struct node *var_node_tail=NULL;
/*var_node_head[]: an array for the description of the constraints*/

struct node *constraint_node=NULL;
/*constraint_node[]: an array for constraints;*/

int *array_two_variable_constraint=NULL;
/*
array_two_variable_constraint[][]: an array of 
** two-variable-contraints.
*/

int *count_two_variable_constraint=NULL;
/*count_two_variable_constraint[][]: an array of the count of 
** two-variable-contraints.
*/

struct two_values domain_of_two_variable_constraint[size1][size1];

ElemId *array_lub=NULL ;
/*array_lub[][]:an array of least-upper-bound constraints.
*/

int *count_unit_lub=NULL;
/*count_unit_lub[]:a count for the units in a upper-bound-constraints; */

int *type_array_lub=NULL;
/*type_array_lub[][]: type of the units in upper-bound-constraints. 
** type_array_lub[][]=0: constant; type_array_lub[][]=1: variable
*/

int *reference_unit_lub=NULL;
/*reference_unit_lub[][]: the reference in domain[] or variables[] to the units of lub
*/

int *number_variable_lub=NULL;
/*number_variable_lub[]: an array for the number of variables in a 
** least-upper-bound-constraint. 
*/

int *vector;
/* vector[count_variable][count_constant]: an array of solution_set for 
** variables.
*/
 
int *solutions=NULL;
/*solutions[][]: an array for the solvings.*/
  
int count_solution;
/*count_solution: a count for the solvings;*/

int count_constant=0;
/*count_constant: a count for the constants; */

int count_variable=0;
/*count_variable: a count for the variables; */

int number_binary_constraint=0;
/*number_binary_constraint: a count for the binary constraints;*/

int number_lub=0;
/*number_lub: the number of the least_upper-bound-constraints; 
*/

int number_order=0;
/*number_order: a count for the order-relations*/

int number_binary_variable_constraint=0;
/*number_binary_variable_constraint: a count for the constraints 
** with two variables
*/

int number_constraint_ee=0;
int number_constraint_ev=0;	
int number_constraint_ve=0;	
int number_constraint_vv=0;
/*
** number_constraint_ee: a count for the Constraint-ee 
** number_constraint_ev: a count for the Constraint-ev 
** number_constraint_ve: a count for the Constraint-ve 
** number_constraint_vv: a count for the Constraint-vv 
*/
	
int count_lub_constraint=0; 
/*count_lub_constraint: a count for least-upper-bound-constraints 
*/
 	
int count_lub_unit=2;
/*count_lub_unit: a count for the units in a least-upper-bound-constraint 
*/
 
int size_solution=size3;
/* a size of the array for solutions[] */



void set_null_elem(ElemId x)
{
#ifdef PRINT 
  printf("set_null_elem( %lu \blu ); \n",x);
#endif 

	NULL_Elem = x;
}


char *
srealloc(char *ptr,unsigned size)
{
	char  *result;

	if((result = realloc(ptr,size))==(char *)0)
	{
		iftrace(fprintf(stderr,"\ncannot realloc,size= %d bytes\n", size));
		return(NULL);
	}
	iftrace(printf("\n realloc, size= %d bytes\n", size));
	return(result);
}
	


/* Interface between the Prolog-Program and CSP */

long get_count_variable(void)
{
#ifdef PRINT 
  printf("get_count_variable(); \n");
#endif 
   return((long)count_variable); 
}
 
long get_count_solution(void)
{
#ifdef PRINT 
  printf("get_count_solution(); \n"); 
#endif 
 return((long)count_solution);

}
 
CVarId  get_variables(long index)
{
#ifdef PRINT 
  printf("get_variables( %ld ); \n",index); 
#endif
  return((index < count_variable) ? variables[index] : 0); 
}
 
ElemId get_solutions(long index1, long index2)
{
#ifdef PRINT 
  printf("get_solutions( %ld %ld );\n",index1,index2); 
#endif
 return( ((index2 < count_variable ) && (index1 < count_solution)) ? domain[solutions[index1*count_variable+index2]] : 0)
;
 
}
 
ElemId get_newvalue(long index)
{
#ifdef PRINT 
  printf("get_newvalue( %ld );\n", index); 
#endif
   return( newvalue[index] );

}
 
ElemId get_neighbor(long index)
{
#ifdef PRINT 
  printf("get_neighbor( %ld ); \n",index); 
#endif
     return( neighbor[index] );
 
}


void constant_num(int number)
{
#ifdef PRINT 
  printf("constant_num(%i); \n",number); 
#endif
	domain=(ElemId*)calloc(number,sizeof(ElemId));
	
}



void constant(ElemId e)
{
	int id;

#ifdef PRINT 
  printf("constant((ElemId) %lu \blu);\n",(long)e); 
#endif
	id=count_constant;
	domain[id]=e;
	count_constant++;
}


/* Routinen zur Definition der geordneten Struktur */

void order_num(int number)
{
#ifdef PRINT 
  printf("order_num(%i); \n",number); 
#endif
	order1=(ElemId *)calloc(number,sizeof(ElemId));
	order2=(ElemId *)calloc(number,sizeof(ElemId));
}

void order_def_less(ElemId a, ElemId b)
/* es gelte a =< b */
{
	int io;

#ifdef PRINT 
  printf("order_def_less((ElemId) %lu \blu, (ElemId) %lu \blu); \n",a,b); 
#endif
	io=number_order;	
	order1[io]=a;
	order2[io]=b;
	number_order++;	
}


/* Routinen zur Definition der Constraints */

void variable_num(int number)
{
#ifdef PRINT 
  printf("variable_num(%i); \n",number); 
#endif

	variables=(CVarId *)calloc(number,sizeof(CVarId));
}

void variable(CVarId x)
{
	int iv;

#ifdef PRINT 
  printf("variable((CVarId) %lu \blu); \n",x); 
#endif

	iv=count_variable;
	variables[iv]=x;
	count_variable++;
}


/* binary constraints */

void binary_constraint_num(int number)
{
#ifdef PRINT 
  printf("binary_constraint_num(%i); \n",number); 
#endif

	number_binary_constraint=number;
	ee1=(ElemId *)calloc(number,sizeof(ElemId  ));
	ee2=(ElemId *)calloc(number,sizeof(ElemId  ));
	ev1=(ElemId *)calloc(number,sizeof(ElemId  ));
	ev2=(ElemId *)calloc(number,sizeof(ElemId  ));
	ve1=(ElemId *)calloc(number,sizeof(ElemId  ));
	ve2=(ElemId *)calloc(number,sizeof(ElemId  ));
	vv1=(ElemId *)calloc(number,sizeof(ElemId  ));
	vv2=(ElemId *)calloc(number,sizeof(ElemId  ));
}

void constraint_def_ee(ElemId a, ElemId b)  /* a =< b */
{
	int iee;

#ifdef PRINT 
  printf("constraint_def_ee((ElemId) %lu \blu,(ElemId) %lu \blu); \n",a,b); 
#endif

	iee=number_constraint_ee;
	ee1[iee]=a;
	ee2[iee]=b;
	number_constraint_ee++; /*number_constraint_ee: a count for the Constraint-ee */
}
	
void constraint_def_ev(ElemId a, CVarId v)  /* a =< v */
{
	int iev;

#ifdef PRINT 
  printf("constraint_def_ev((ElemId) %lu \blu,(CVarId) %lu \blu); \n",a,v); 
#endif

	iev=number_constraint_ev;
	ev1[iev]=a;
	ev2[iev]=v;
	number_constraint_ev++; /*number_constraint_ev: a count for the Constraint-ev */

}

void constraint_def_ve(CVarId u, ElemId b)  /* u =< b */
{
	int ive;

#ifdef PRINT 
  printf("constraint_def_ve((CVarId) %lu \blu,(ElemId) %lu \blu); \n",u,b); 
#endif
	ive=number_constraint_ve;
	ve1[ive]=u;
	ve2[ive]=b;
	number_constraint_ve++; /*number_constraint_ve: a count for the Constraint-ve */
}

	
void constraint_def_vv(CVarId u, CVarId v)  /* u =< v */
{
	int ivv;

#ifdef PRINT 
  printf("constraint_def_vv((CVarId) %lu \blu,(CVarId) %lu \blu); \n",u,v); 
#endif
	ivv=number_constraint_vv;
	vv1[ivv]=u;
	vv2[ivv]=v;
	number_constraint_vv++; /*number_constraint_vv: a count for the Constraint-vv */
}



/* upper-bound-constraint der Art
       v = a \ub b \ub u \ub v ...
   muessen aus den einzelnen Elementen zusammengebaut werden 
   (\up steht fuer den upper-bound Operator) 
*/

void upper_bound_constraint_num(int number)  /*the number of  upper-bound-constraints*/
{
#ifdef PRINT 
  printf("upper_bound_constraint_num(%i); \n",number); 
#endif 

	number_lub=number;
	count_unit_lub=(int*)calloc(number_lub,sizeof(int));
	array_lub=(ElemId  *)calloc(number_lub*(count_constant+count_variable),sizeof(ElemId  ));
	type_array_lub=(int*)calloc(number_lub*(count_constant+count_variable),sizeof(int));


}


UbExpr ub_first_e(ElemId a)
{
	int i1;

#ifdef PRINT 
  printf("ub_first_e((ElemId) %lu \blu); \n",a); 
#endif
	
	i1=count_lub_constraint;
	array_lub[i1*(count_constant+count_variable)+1]=a;
	type_array_lub[i1*(count_constant+count_variable)+1]=0; /* type_=0: constant */

	return(NULL);
}
	
UbExpr ub_first_v(CVarId u)
{
	int j1;

#ifdef PRINT 
  printf("ub_first_v((CVarId) %lu \blu); \n",u); 
#endif
	
	j1=count_lub_constraint;	
	array_lub[j1*(count_constant+count_variable)+1]=u;
	type_array_lub[j1*(count_constant+count_variable)+1]=1; /* type_=1: variable */
	return(NULL);

}

UbExpr ub_expr_e(UbExpr u, ElemId a) /* return: u \lub a */
{
	int i2;
#ifdef PRINT 
  printf("ub_expr_e(0,(ElemId) %lu \blu); \n",a); 
#endif

	
	i2=count_lub_constraint;
	array_lub[i2*(count_constant+count_variable)+count_lub_unit]=a;
	type_array_lub[i2*(count_constant+count_variable)+count_lub_unit]=0; /* type_=0: constant */
	
	count_lub_unit++;
	return(NULL);

}


UbExpr ub_expr_v(UbExpr u, CVarId v) /* return: u \lub v */
{
	int j2;

#ifdef PRINT 
  printf("ub_expr_v(0, (CVarId) %lu \blu); \n",v); 
#endif

	j2=count_lub_constraint;
	array_lub[j2*(count_constant+count_variable)+count_lub_unit]=v;
	type_array_lub[j2*(count_constant+count_variable)+count_lub_unit]=1; /* type_=1: variable */
	count_lub_unit++;

	return(NULL);

}


void constraint_def_eu(ElemId a, UbExpr u)  /* a = u */
{
	int i;

#ifdef PRINT 
  printf("constraint_def_eu((ElemId) %lu \blu, 0); \n",a); 
#endif

	i=count_lub_constraint;
	array_lub[i*(count_constant+count_variable)+0]=a;
	type_array_lub[i*(count_constant+count_variable)+0]=0; /* type_=0: constant */
	count_unit_lub[i]=count_lub_unit;
	count_lub_constraint++;
	count_lub_unit=2;
	

}
	

void constraint_def_vu(CVarId v, UbExpr u)  /* v = u */
{
	int j;

#ifdef PRINT 
  printf("constraint_def_vu((CVarId) %lu \blu,0); \n",v); 
#endif

	j=count_lub_constraint;
	array_lub[j*(count_constant+count_variable)+0]=v;
	type_array_lub[j*(count_constant+count_variable)+0]=1; /* type_=1: variable */
	count_unit_lub[j]=count_lub_unit;
	count_lub_constraint++;
	count_lub_unit=2;

}




/*
struct node *ds_head(ElemId p):  search head_node and creat the head_linkink,
p: a pointer to the working node,
return a pointer of the next node in the head_linkink 
*/	

struct node 
*ds_head(ElemId p)		
		/* search head_node and creat the head_linkink*/
{
	struct node *q,*s,*t;
	int i;

	q=NULL;
	s=NULL;
	for (i=0;i<number_order;i++)	
	  if(p==order_node[i].head)
	  {	
	 	q=&order_node[i];
		t=s;
		s=q;
		order_node[i].head_link=t;
	  }

	return(q); /*return a pointer of the next node in the head_linkink*/
}



/*
struct node 
*ds_tail(ElemId p): search tail_node and creat the tail_linkink;
p: a pointer to the working node,
return a pointer of the next node in the tail_linkink
*/

struct node 
*ds_tail(ElemId p)		/*search tail_node and creat the tail_linkink*/
{
	struct node *q,*s,*t;
	int i;

	
	q=NULL;
	s=NULL;
	for (i=0;i<number_order;i++)
	  if(p==order_node[i].tail)
	  {
	 	q=&order_node[i];
		t=s;
		s=q;
		order_node[i].tail_link=t;
	  }

	return(q);
}



/*
relation_graph(): creat the data-structure of relation strings, no return value.
*/	   
			
void relation_graph()
{
       	ElemId dtmp;
	int i;
       	 
	order_node=(struct node *)calloc(number_order,sizeof(struct node));
	dom_node_head=(struct node *)calloc(count_constant,sizeof(struct node));
	dom_node_tail=(struct node *)calloc(count_constant,sizeof(struct node));
       
        for (i=0;i<number_order; i++)
        {
          order_node[i].head = order1[i];                  
          order_node[i].tail = order2[i];	/* creat relation node */
          order_node[i].head_link = NULL;
          order_node[i].tail_link = NULL;        
        }
 
        for (i=0; i<count_constant;i++)
	{
	  dom_node_head[i].head=domain[i];	/*dom_node_head:domain_node_head_link*/
	  dom_node_head[i].tail=-1;	
	  dtmp=dom_node_head[i].head;
	  dom_node_head[i].head_link=ds_head(dtmp); 
	  dom_node_head[i].tail_link=NULL;

	  dom_node_tail[i].tail=dtmp;	/*dom_node_tail:dom_node_tail_link*/
	  dom_node_tail[i].head=-1;
	  dom_node_tail[i].head_link=NULL;
	  dom_node_tail[i].tail_link=ds_tail(dtmp);
	}

	  
}


/*
** ElemId repeat(ElemId A): check whether A has same name as one in domain[] 
** or variables[], if yes, then modify A=A+domain[0]; if no, return:A.
*/
ElemId repeat(ElemId A)
{
	int i;

	for(i=0;i<count_constant;i++)
		if(domain[i] ==A)
			goto modify;


	if(A == NULL_Elem )
		goto modify;


	return(A); /* no modification */

 modify: A=A+domain[0];

	return(repeat(A));

}

	
	


/*
** int check_unique(): check whether those least-upper-bounds in 
** the given order-relations are unique, if no, print out some
** information and change the given order-relations with some new 
** constants into equivelent order-relations.
** return:0,unique; othercount_lub_unitse return:n, the number of new added constants.
*/

int check_unique()
{

	int i, j, count_of_tail, count_of_head, num_head,m1,new_num_dom,dom_size;
	int count_new_constant,k,l,num_newvalue,k1,id;
	ElemId   tailvalue, headvalue,temp,tmp_new,*head=NULL;
	struct node *p, *q, *p1,*ptr,*qtr;
	
#ifdef PRINT 
  printf("check_unique();\n"); 
#endif
	
	relation_graph();


/*
** At first, check all constants one by one to see if all least 
** upper bounds are unique.
*/



	

	/* work with the tail_linkink of a constant*/

	new_order_node=(struct node *)calloc(size1, sizeof(struct node));
	head=(ElemId *)calloc(size1, sizeof(ElemId));
	newvalue=(ElemId *)calloc(size1, sizeof(ElemId));

	new_num_dom=count_constant;
	dom_size=count_constant;
	count_of_tail=0; 	
	count_new_constant=0;
	num_newvalue=0;	

	for(i=0;i<count_constant;i++)
	{/*A*/

	  count_of_head=0;	/*hc: a count of head[]*/

	  tailvalue=dom_node_tail[i].tail;
	  p=dom_node_tail[i].tail_link;

	  if(p==NULL)	/*no children*/
		goto cu1;

	  
	  

	  q=p->tail_link;
		
	  if(q==NULL)	/*only one child*/
		goto cu1;

	  head[count_of_head]=p->head;
	  count_of_head++;
	  p=q;

	  while(p!=NULL) /*take all children into head[] */
		{
			q=p->tail_link;
			head[count_of_head]=p->head;
			count_of_head++;
			p=q;
		}

	num_head=count_of_head;	/*hcount: the number of head[]*/

	/* work with the head_linkink of every head[] to check if there is another node like a upper bound of head[]*/
		

	 for(j=0;j<num_head;j++)
	 {/*D*/
		id=j;
		for(k=0;k<count_constant;k++)
			if(head[j]==dom_node_head[k].head)
				goto cu2;

	   cu2: p=dom_node_head[k].head_link;
			
		while(p!=NULL)
		{/*F*/

			temp=p->head;
			if(p->tail==tailvalue)
			{
				
				p=p->head_link;
				if(p==NULL)
					goto cu3;
			}
			q=p->head_link; 
			if(q==NULL)
				goto cu3;

			for(k1=0;k1<count_constant;k1++)
				if(p->tail == dom_node_tail[k1].tail)
					goto cu21;

		cu21:   p=dom_node_tail[k1].tail_link;
			
			  
			while(p!=NULL)
			{/*G*/

				  headvalue=p->head;

				  for(l=id+1;l<num_head;l++)
				    if(headvalue==head[l])	/*here is one!*/
				     {/*H*/
					
					id=l;

					p1=q;
					while(p1->tail != tailvalue)
					{
					  p1=p1->head_link;
					  if(p1 == NULL)
						goto cu3;;
					}


					printf("\n\tNOTE:\nthe least-upper-bound in the order-relation is not unique. \nNow some new constants are added.\n");

					domain=(ElemId *)srealloc((char *)domain, sizeof(ElemId)*(dom_size+1));
					if(domain==NULL)
						return(-1);
					dom_node_head=(struct node *)srealloc((char  *)dom_node_head,sizeof(struct node)*(dom_size+1));
					if(dom_node_head==NULL)
						return(-1);

					dom_node_tail=(struct node *)srealloc((char  *)dom_node_tail,sizeof(struct node)*(dom_size+1));
					if(dom_node_tail==NULL)
						return(-1);
					
					dom_size++;

	/*create a new node in order to make the upper bound unique*/

tmp_new=repeat(headvalue+p->tail);

for(m1=0;m1<num_newvalue;m1++)
	if(newvalue[m1] ==tmp_new)
		tmp_new=tmp_new+domain[0]; 
	
newvalue[num_newvalue]=tmp_new;
					new_order_node[count_new_constant].head=headvalue;
										
					new_order_node[count_new_constant].tail=newvalue[num_newvalue];
					new_order_node[count_new_constant].tail_link=NULL;
					new_order_node[count_new_constant].head_link=p->head_link;
					

new_order_node[count_new_constant+1].head=temp;
					new_order_node[count_new_constant+1].tail=newvalue[num_newvalue];
					new_order_node[count_new_constant+1].tail_link=&new_order_node[count_new_constant];
					new_order_node[count_new_constant+1].head_link=p1->head_link;


					
					p->head=newvalue[num_newvalue];
					p->head_link=p1;


					p1->head=newvalue[num_newvalue];
					p1->head_link=NULL;
					


		/*change the structure of order relation*/

						
					domain[new_num_dom]=newvalue[num_newvalue];

					dom_node_head[new_num_dom].head=newvalue[num_newvalue];
					dom_node_head[new_num_dom].tail=-1;
					dom_node_head[new_num_dom].head_link=p;
					dom_node_head[new_num_dom].tail_link=NULL;

					dom_node_tail[new_num_dom].tail=newvalue[num_newvalue];
					dom_node_tail[new_num_dom].head=-1;
					dom_node_tail[new_num_dom].head_link=NULL;
					dom_node_tail[new_num_dom].tail_link=&new_order_node[count_new_constant+1];

					new_num_dom++;

					
                                      
									
					for(m1=0;m1<new_num_dom;m1++)					
					 if(dom_node_head[m1].head==temp)
					  {
					    if(dom_node_head[m1].head_link==p1)
						{
						dom_node_head[m1].head_link=&new_order_node[count_new_constant+1];
						goto cu5;
						}
					    
					    qtr=dom_node_head[m1].head_link;
					   while(qtr->head_link!=p1)
						qtr=qtr->head_link;
						
						 qtr->head_link=&new_order_node[count_new_constant+1];
						goto cu5;
					 }

				cu5:	for(m1=0;m1<new_num_dom;m1++)
						if(dom_node_head[m1].head==headvalue)
						{
					    if(dom_node_head[m1].head_link==p)
						{
						dom_node_head[m1].head_link=&new_order_node[count_new_constant];
						goto cu6;
						}
					    
					    ptr=dom_node_head[m1].head_link;
					   while(ptr->head_link!=p)
						ptr=ptr->head_link;
					ptr->head_link=&new_order_node[count_new_constant];						goto cu6;
						} 






				cu6:	printf("\n the new constant is %lu\n",newvalue[num_newvalue]);

					count_new_constant=count_new_constant+2;
					num_newvalue++;

					
					goto cu3;
				   }/*H, two new nodes added*/

				p=p->tail_link;


			  }/*G, next node in tail_linkink*/

			  p=q;

		   }/*F, go on next node in head_linkink*/
			      

	  
	cu3:				

					
					

	     }/*D, next node of head[]*/


     cu1: 	/* go on next constant in dom_node_tail[]*/

	} /*A*/

	count_constant=new_num_dom;
	
	free(head);

	if (count_new_constant>=1)
	{
	 printf("\n the number of new added constants is %d\n", num_newvalue);
	 return(num_newvalue);
	}
	else
	{
		/*printf("\n The given order-relation is unique\n");*/
		return(0); /* it is unique */
	}


}

/*
** int less_neighbor(ElemId x):To search the direct neighbor:y of x, 
** with y<=x, x!=y, y is maximal.
** return: n, the number of neighbors;
** return: 0, no neighbor. return:-1, error.
*/

int less_neighbor(ElemId x)
{
	int i,j;	
	struct node *pointer1;

#ifdef PRINT 
  printf("less_neighbor((ElemId) %lu \blu); \n",x); 
#endif 	
	neighbor=(ElemId *)calloc(count_constant,sizeof(ElemId));
	
	for(j=0;j<count_constant;j++)
	   if(dom_node_tail[j].tail==x)
		goto ne1;

	printf("\n error in less_neighbor,\t  x=%ld not in Domain\n",x);
	return(-1);
	
   ne1:	i=0;
	if(dom_node_tail[j].tail_link==NULL)
		return(0);
	else
	{
		pointer1=dom_node_tail[j].tail_link;
		neighbor[i]=pointer1->head;
		i++;
		pointer1=pointer1->tail_link;
		while(pointer1!=NULL)
		{
			neighbor[i]=pointer1->head;
			i++;
			pointer1=pointer1->tail_link;
		}
		neighbor[i]=-1;
		return(i);
	}
}			



/*
struct node *vs_head(CVarId p):  search constrait_head_node and creat the constrait_head_linkink,
p: a pointer to the working node,
return a pointer of the next node in the constrait_head_linkink 
*/	

struct node 
*vs_head(CVarId p)		
	/* search head_node and creat constrait_head_linkink*/
{
	struct node  *q,*s,*t;
	int i;

	q=NULL;
	s=NULL;
	for (i=0;i<number_constraint_ev+number_constraint_ve;i++)
	    if(p==constraint_node[i].head)
	    {
	 	q=&constraint_node[i];
		t=s;
		s=q;
		constraint_node[i].head_link=t;
	    }
	return(q);
}

/*
struct node *vs_tail(CVarId p):  search constrait_tail_node and creat the constrait_tail_linkink,
p: a pointer to the working node,
return a pointer of the next node in the constrait_tail_linkink 
*/	

struct node 
*vs_tail(CVarId p)	/*search tail_node and creat constrait_ tail_linkink*/
{
	struct node *q,*s,*t;
	int i;

	q=NULL;
	s=NULL;
	for (i=0;i<number_constraint_ev+number_constraint_ve;i++)
	  if(p==constraint_node[i].tail)
	  {
	 	q=&constraint_node[i];
		t=s;
		s=q;
		constraint_node[i].tail_link=t;
	  }
	return(q);
}



/*
constraint_graph(): creat the data-structure of constraints, no return value.
*/	

void constraint_graph()
{
	CVarId vtmp, t, h;
	int i,j,k,c;

	constraint_node=(struct node *)calloc((number_constraint_ev+number_constraint_ve),sizeof(struct node));
	var_node_head=(struct node *)calloc(count_variable,sizeof(struct node));
	var_node_tail=(struct node *)calloc(count_variable,sizeof(struct node));
                  
        for (i=0;i<number_constraint_ev; i++)
        {
          constraint_node[i].head = ev1[i];                 
          constraint_node[i].tail = ev2[i];	/* creat constraint node */
          constraint_node[i].head_link = NULL;
          constraint_node[i].tail_link = NULL;        
        }


        for (j=i;j<number_constraint_ev+number_constraint_ve; j++)
        {
          constraint_node[j].head = ve1[j-i];                 
          constraint_node[j].tail = ve2[j-i];	/* creat constraint node */
          constraint_node[j].head_link = NULL;
          constraint_node[j].tail_link = NULL;        
        }



	/* creat a binary constraint table array_two_variable_constraint[]*/

	array_two_variable_constraint=(int *)calloc(number_constraint_vv*2,sizeof(int));

	count_two_variable_constraint=(int *)calloc(number_constraint_vv, sizeof(int));	


	c=0;
	for(i=0;i<number_constraint_vv;i++)
	{
		h=vv1[i];
		t=vv2[i];		
	 	for(j=0;j<count_variable;j++)
		  if(t == variables[j])
			for(k=0;k<count_variable;k++)
			  if(h == variables[k])
			   {
				array_two_variable_constraint[c*2+0]=j;  /*array_two_variable_constraint[][0]:tail*/

				array_two_variable_constraint[c*2+1]=k;  /*array_two_variable_constraint[][1]:head*/
			  	
				c++;
			   }
	}
	number_binary_variable_constraint=c; /* number_binary_variable_constraint: the number of the binary constraints */
	


			  			  

 
        for (i=0; i<count_variable;i++)
	{
	  var_node_head[i].tail=-1;		/*var_node_head:variab_node_head_link*/
	  var_node_head[i].head=variables[i];

	  vtmp=var_node_head[i].head;

	  var_node_head[i].head_link=vs_head(vtmp);

	  var_node_head[i].tail_link=NULL;

	  var_node_tail[i].tail=variables[i];	/*var_node_tail:variab_node_tail_link*/
	  var_node_tail[i].head=-1;
	  var_node_tail[i].head_link=NULL;
	  var_node_tail[i].tail_link=vs_tail(vtmp);
	}

  

 }



	



/* link_value(ElemId value1, ElemId value2): (value1<=value2)?check whether there is a link from value1(head) to value2(tail) in order_nodes. success:return 1; fail:return 0; error:return -1. */  


int link_value(ElemId value1, ElemId value2)
{
	/*is there a link from value1(head) to value2(tail) in order_nodes?*/
	/* work with the head_linkink of value1*/
	/*success:return 0; fail:return 1; error:return -1*/

	struct node *p,*s;
	ElemId q;
	int i, r1;


        iftrace(printf("\nlink <%d,%d>\n",value1,value2));


	if (value1==value2)
	{

	iftrace(printf("\n success link0 \n"));

		return(1);	/* value1=value2: satisfy */

	}


	for(i=0;i<count_constant;i++)
	  if(value1==dom_node_head[i].head)
		goto t6;


	printf("\n error No.1 in link_value\t ElemId value1=%lu is not in Domain \n",value1);
	return(-1);


	
    t6: p=dom_node_head[i].head_link;
	if(p==NULL)
	{

	iftrace(printf("\n fail link1 \n"));

	 return(0);
	}
	
	q=p->tail;
	s=p->head_link;
    t7: if (q==value2) /*q=value2:The link from value1 to value2 exist*/
	{

	iftrace(printf("\n success link1 \n"));

	 return(1);
	}



	r1=link_value(q,value2);
	if(r1==0)
	  {
	    if(s!=NULL)
	     {
		p=s;
		q=p->tail;
		s=p->head_link;
		goto t7;
	     }

	    iftrace(printf("\n fail link2 \n"));
	    return(0);
	  }

	  
	if(r1==1)
	{

	iftrace(printf("\n success link2 \n"));

	  return(1);
	}


	printf("\n error No.2 in link_value\n");


	
	  return(-1);
	
}


/* int test_leq(ElemId x, ElemId y):  check whether there exist the 
** order x =< y in the given order-relations; 
** return: 1, if yes; return: 0, if no; return:-1, error.
*/
int test_leq(ElemId x, ElemId y)
{
	int i,f;

#ifdef PRINT 
  printf("test_leq((ElemId) %lu \blu,(ElemId) %lu \blu); \n",x,y); 
#endif
	for(i=0;i<number_order;i++)
		if(x==order1[i] && y== order2[i])
			return(1);

	f=link_value(x, y);
	if(f==-1)
		return(-1);
	else
		if(f==1)
			return(1); 
		else
			return(0);


}

/* int is_maximal( ElemId a);
** return: 1, if a is the maximal element in the given order relation, 
** 	      i.e. if a<=x, then a=x.
** return: 0, if  not.
*/
int is_maximal( ElemId a)
{
	int i;

#ifdef PRINT 
  printf("is_maximal((ElemId) %lu \blu); \n",a); 
#endif

        for(i=0;i<count_constant;i++)
                if(a==dom_node_head[i].head)
			goto jm;

   jm:	if(dom_node_head[i].head_link==NULL)
		return(1);	/*a is maximal*/
	else
		return(0);	/*a is not maximal*/
	


}


/*int is_minimal(ElemId a);
** return: 1, if a is the minimal element in the given order relation,
**		i.e. if x>=a , then x=a.
** return: 0, if not.
*/
int is_minimal(ElemId a)
{
	int i;

#ifdef PRINT 
  printf("is_minimal((ElemId) %lu \blu); \n",a); 
#endif
        for(i=0;i<count_constant;i++)
                if(a==dom_node_tail[i].tail)
			goto im;

   im: if(dom_node_tail[i].tail_link==NULL)
		return(1);	/*a is minimal*/
	else
		return(0);	/*a is not minimal*/


}


/* ElemId glb(ElemId a, ElemId b);
** calculate the greatest-low-bound of element a and b,
** return: ElemId c, if c =a \lb b	
** return: NULL_Elem, if no c exist.
*/
ElemId glb(ElemId a, ElemId b)
{

	 
	int i, j, ia, ib,ju1,ju2, tc, hc, hcount,sc,imax,f1,f2;
	int scount;
	ElemId  result,value2,value1,u1,u2,r1,r2;
	struct node *p, *q, *p1,*q1,*p2,*q2;
	ElemId  *same_child=NULL;
	ElemId  *child_head=NULL;

#ifdef PRINT 
  printf("glb((ElemId) %lu \blu,(ElemId) %lu \blu); \n",a,b); 
#endif


	tc=0;	/*tc: a count of tail[]*/
	ia=-1; /* ia: a count for the position of a in dom_node_tail[]*/
	ib=-1; /* ib: a count for the position of b in dom_node_tail[]*/

	f1=link_value(a,b);
	if(f1==1)
		return(a);	/*a<=b*/

	f2=link_value(b,a);
	if(f2==1)
		return(b);	/*b<=a*/

	for(i=0;i<count_constant;i++)
	 {
	  	if(a==dom_node_tail[i].tail)
	   	{
			ia=i; 
   			if(dom_node_tail[ia].tail_link==NULL)
				return(NULL_Elem);	/*no child of a*/
	   	}

	  	if(b==dom_node_tail[i].tail)
	  	{
			ib=i; 
   			if(dom_node_tail[ib].tail_link==NULL)
				return(NULL_Elem);	/*no child of b*/
		
		}

	}

	

	/* work with the tail_linkink of a*/

 	child_head=(ElemId*)calloc(count_constant, sizeof(ElemId));
	same_child=(ElemId*)calloc(count_constant, sizeof(ElemId));
 
	  hc=0;	/*hc: a count of child_head[]*/


	  p=dom_node_tail[ia].tail_link;
	  q=p->tail_link;

	  child_head[hc]=p->head;
	  hc++;
	  p=q;

	  while(p!=NULL) /*take all children of a into child_head[] */
		{
			q=p->tail_link;
			child_head[hc]=p->head;
			hc++;
			p=q;
		}

	  hcount=hc;	/*hcount: the number of child_head[]*/

	iftrace(printf("\n hcount=%d\n",hcount));

	

	/* work with the tail_linkink of b*/
	
	p1=dom_node_tail[ib].tail_link;
	q1=p1->tail_link;
	sc=0; /*sc: a count of same_child[]*/


	/*compare the children of a and b*/
	
        for(i=0;i<hcount;i++)
	{

	   	u1=p1->head;
		u2=child_head[i];

                if(p1->head==child_head[i])
                {
                        same_child[sc]=child_head[i];
                        sc++;
                }
		else
			{
				f1=link_value(p1->head,child_head[i]);
				if(f1==1)
				{
					same_child[sc]=p1->head;
                                	sc++;

				}
				else
				{
					f2=link_value(child_head[i],p1->head);
					if(f2==1)
					{
						same_child[sc]=child_head[i];
                                		sc++;

					}
					else
					{
						if(is_minimal(child_head[i])!=1 && is_minimal(p1->head) !=1)
						{
						   	for(j=0;j<count_constant;j++)
							 {
	  						   if(u1==dom_node_tail[j].tail)
								ju1=j;
							   if(u2==dom_node_tail[j].tail)
								ju2=j;
							}
							p2=dom_node_tail[ju1].tail_link;
							r1=p2->head;
							q2=dom_node_tail[ju2].tail_link;
							r2=q2->head;
							same_child[sc]=glb(r1,r2);
                                			sc++;
						}
					}

							

				}
			}

	}

        p1=q1;

        while(p1!=NULL)
        {
                q1=p1->tail_link;
		u1=p1->head;
		u2=child_head[i];

                for(i=0;i<hcount;i++)
                        if(u1==u2)
                        {
                                same_child[sc]=child_head[i];
                                sc++;
                        }
			else
			{
				f1=link_value(p1->head,child_head[i]);
				if(f1==1)
				{
					same_child[sc]=p1->head;
                                	sc++;
				}
				else
				{
					f2=link_value(child_head[i],p1->head);
					if(f2==1)
					{
						same_child[sc]=child_head[i];
                                		sc++;

					}
					else
					{
						if(is_minimal(child_head[i])==1 && is_minimal(p1->head) ==1)
						{
						   	for(j=0;j<count_constant;j++)
							 {
	  						   if(u1==dom_node_tail[j].tail)
								ju1=j;
							   if(u2==dom_node_tail[j].tail)
								ju2=j;
							}
							p2=dom_node_tail[ju1].tail_link;
							r1=p2->head;
							q2=dom_node_tail[ju2].tail_link;
							r2=q2->head;
							same_child[sc]=glb(r1,r2);
                                			sc++;
						}
					}


				}
			}	
                p1=q1;
        }

        scount=sc; /*scount: a count of same_child[]*/

	iftrace(printf("\n scount=%d\n",scount));


        if(scount==0) /* no common children, no greatest low bound*/
	{
		if(child_head != NULL)
		{
			free(child_head);
			child_head=NULL;
		}
		if(same_child != NULL)
		{
			free(same_child);
			same_child=NULL;
		}
		return(NULL_Elem);
	}
		
                     

        if(scount==1)  /*The glb of a and b is found*/
	{
		result=same_child[0];
		if(child_head != NULL)
		{
			free(child_head);
			child_head=NULL;
		}
		if(same_child != NULL)
		{
			free(same_child);
			same_child=NULL;
		}
                return(result);
	}

        /*scount>=2, more than two constants are same, check if unique? */

        imax=0;
        for(j=1;j<scount;j++)
        {

	iftrace(printf("\n j=%d, imax=%d\n",j,imax));
             value1=same_child[imax];
             value2=same_child[j];

             f1=link_value(value1,value2);

             if(f1==1)	/*value1<=value2*/
		imax=j;

	     else		
              {
                f2=link_value(value2,value1);

                if(f2==0)
                        return(0); /*there are least two noninterventist glb*/

              } /*value2<=value1*/

 

        }
		result=same_child[imax];
		if(child_head != NULL)
		{
			free(child_head);
			child_head=NULL;
		}
		if(same_child != NULL)
		{
			free(same_child);
			same_child=NULL;
		}
                return(result);
	
 

}


/* ElemId lub(ElemId a, ElemId b)
** calculate the least upper bound of element a and b,
** return: ElemId c, if c =a \lub b	
** return: NULL_Elem, if no c exist.
*/

ElemId lub(ElemId a, ElemId b)
{

        int i, j, ia, ib,ju1,ju2, tc, tcount,sp,imin,f1,f2;
        int scount,count_head;
        ElemId  result,value2,value1,u1,u2,r1,r2;
        struct node *p, *q, *p1,*q1,*p2,*q2;
        ElemId  *same_parent=NULL;
        ElemId  *parent_tail=NULL;

#ifdef PRINT 
  printf("lub((ElemId) %lu \blu,(ElemId) %lu \blu); \n",a,b); 
#endif

        count_head=0;   /*count_head: a count of head[]*/
        ia=-1; /* ia: a count for the position of a in dom_node_head[]*/
        ib=-1; /* ib: a count for the position of b in dom_node_head[]*/

	if(a==b)
		return(a);	/*a=b*/

        f1=link_value(a,b);
        if(f1==1)
                return(b);      /*a<=b*/

        f2=link_value(b,a);
        if(f2==1)
                return(a);      /*b<=a*/

        for(i=0;i<count_constant;i++)
         {
                if(a==dom_node_head[i].head)
                {
                        ia=i; 
                        if(dom_node_head[ia].head_link==NULL)
                                return(NULL_Elem);      /*no parent of a*/
                }

                if(b==dom_node_head[i].head)
                {
                        ib=i; 
                        if(dom_node_head[ib].head_link==NULL)
                                return(NULL_Elem);      /*no parent of b*/
                
                }

         }

        /* work with the head_linkink of a*/

        parent_tail=(ElemId*)calloc(count_constant, sizeof(ElemId));
        same_parent=(ElemId*)calloc(count_constant, sizeof(ElemId));
 
          tc=0; /*tc: a count of parent_tail[]*/


          p=dom_node_head[ia].head_link;
          q=p->head_link;

          parent_tail[tc]=p->tail;
          tc++;
          p=q;

          while(p!=NULL) /*take all parents of a into parent_tail[] */
                {
                        q=p->head_link;
                        parent_tail[tc]=p->tail;
                        tc++;
                        p=q;
                }

          tcount=tc;    /*tcount: the number of parent_tail[]*/

        iftrace(printf("\n tcount=%d\n",tcount));
      
        /* work with the heag_link of b*/
        
        p1=dom_node_head[ib].head_link;
        q1=p1->head_link;
        sp=0; /*sp: a count of same_parent[]*/


        /*compare the parents of a and b*/
        
        for(i=0;i<tcount;i++)
        {

                u1=p1->tail;
                u2=parent_tail[i];


                if(p1->tail==parent_tail[i])
                {
                        same_parent[sp]=parent_tail[i];
                        sp++;
                }
 
                else
                        {
                                f1=link_value(p1->tail,parent_tail[i]);
                                if(f1==1)
                                {
                                        same_parent[sp]=parent_tail[i];
                                        sp++;
                          }
                                else
                                {
                                        f2=link_value(parent_tail[i],p1->tail);
                                        if(f2==1)
                                        {
                                                same_parent[sp]=p1->tail;
                                                sp++;
                                        }
                                        else
                                        {
                                                if(is_maximal(parent_tail[i])!=1 && is_maximal(p1->tail) !=1)
                                                {
                                                        for(j=0;j<count_constant;j++)
                                                         {
                                                           if(u1==dom_node_head[j].head)                                                                ju1=j;
                                                           if(u2==dom_node_head[j].head)                                                                ju2=j;
                                                        }
                                                        p2=dom_node_head[ju1].head_link;
                                                        r1=p2->tail;
                                                        q2=dom_node_head[ju2].head_link;
                                                        r2=q2->tail;
                                                        same_parent[sp]=lub(r1,r2);
                                                        sp++;
                                                }
                                        }

                                                        

                                }
                        }

        }

        p1=q1;

        while(p1!=NULL)
        {
                q1=p1->head_link;
                u1=p1->tail;
                u2=parent_tail[i];

                for(i=0;i<tcount;i++)
                        if(u1==u2)
                        {
                                same_parent[sp]=parent_tail[i];
                                sp++;
                        }
                        else
                        {
                                f1=link_value(p1->tail,parent_tail[i]);
                                if(f1==1)
                                {
                                        same_parent[sp]=parent_tail[i];
                                        sp++;
                                }
                                else
                                {
                                        f2=link_value(parent_tail[i],p1->tail);
                                        if(f2==1)
                                        {
                                                same_parent[sp]=p1->tail;
                                                sp++;

                                        }
                                        else
                                        {
                                                if(is_maximal(parent_tail[i])==1 && is_maximal(p1->tail) ==1)
                                                {
                                                        for(j=0;j<count_constant;j++)
                                                         {
                                                           if(u1==dom_node_head[j].head)                                                                ju1=j;
                                                           if(u2==dom_node_head[j].head)                                                                ju2=j;
                                                        }
                                                        p2=dom_node_head[ju1].head_link;
                                                        r1=p2->tail;
                                                        q2=dom_node_head[ju2].head_link;
                                                        r2=q2->tail;
                                                        same_parent[sp]=lub(r1,r2);
                                                        sp++;
                                                }
                                        }


                                }
                        }       
                p1=q1;
        }

        scount=sp; /*scount: a count of same_parent[]*/



        if(scount==0) /* no common parents, no least upper bound*/
        {
                if(parent_tail != NULL)
                {
                        free(parent_tail);
                        parent_tail=NULL;
                }
                if(same_parent != NULL)
                {
                        free(same_parent);
                        same_parent=NULL;
                }
                return(NULL_Elem);
        }
                
                     

        if(scount==1)  /*The glb of a and b is found*/
        {
                result=same_parent[0];
                if(parent_tail != NULL)
                {
                        free(parent_tail);
                        parent_tail=NULL;
                }
                if(same_parent != NULL)
                {
                        free(same_parent);
                        same_parent=NULL;
                }
                return(result);
        }

        /*scount>=2, more than two constants are same, check if unique? */

        imin=0;
        for(j=1;j<scount;j++)
        {
             value2=same_parent[imin];
             value1=same_parent[j];

             f1=link_value(value1,value2);

             if(f1==1)  /*value1<=value2*/
                imin=j;

             else               
              {
                f2=link_value(value2,value1);

                if(f2==0)
                        return(NULL_Elem); /*there are least two noninterventist lub*/

              } /*value2<=value1*/

 

        }
                result=same_parent[imin];
                if(parent_tail != NULL)
                {
                        free(parent_tail);
                        parent_tail=NULL;
                }
                if(same_parent != NULL)
                {
                        free(same_parent);
                        same_parent=NULL;
                }
                return(result);
 
 


}


/* check_1(CVarId varp, ElemId valq): check whether <varp,valq> satisfies all constraints in form of constraint_def_ev(ElemId a, CVarId v): a=< v and in form of constraint_def_ve( CVarId u, ElemId b,):u=< b, i.e. satisfies(<x,v>,Cx). varp: a pointer to a variable, valq: a pointer to a value. success:return 1; fail:return 0; error:return -1. */  
	    	   
int check_1(CVarId varp, ElemId valq)
{	

	int i,j,r2,r3;
	struct node *p,*p2,*s1,*s2,*t,*t2,*w2,*w;
	ElemId q, q2, u, u2;


	/* work with head_linkink of the variable*/

	iftrace(printf("\nce1  <varp,valq>=<%d,%d>\n",varp,valq));

	for(i=0;i<count_variable;i++)
	{

	  if(varp==var_node_head[i].head)
	    goto t3;
	}
	printf("\n error in check_1\t CVarId varp=%lu is not in Variable\n",varp);
	return(-1);


    t3: s1=var_node_head[i].head_link;
 

	
    t1:  /* take next constraint_node in the head_linkink of the variable */
       if(s1!=NULL)
	{


	  p=s1;	/*p -> the first node of head_linkink of the variable*/

	  q=p->tail;	/*q -> the tail of a constraint_node in the head_linkink*/

	  s1=p->head_link;	/*s1 -> the next node of the head_linkink */

          iftrace(printf("\nce2 q=%d,p->head=%d\n",q,p->head));


	  goto t211;
	}
	goto tl3;	/* s1= NULL:go to tail_linkink*/

  t211: if(valq!=q)   
	  goto t2;
	
	goto t1;	/* valq=q? if yes,get next constraint in head_linkink */


  	   

   	  
	/* take the constraints which have't two variables<q,valq> */
	/* work with the tail_linkink of order_node_link */

  t2:  for(j=0;j<count_constant;j++)
	{
	 
	 if(q==domain[j])
		  goto t4;
	}
		
	
	printf("\n error in check_1\t ElemId q=%lu is not in Domain\n",q);

	return(-1);

   /*compare valq<=q with all nodes in the tail_linkink of order_node,q  */

    t4:	t=dom_node_tail[j].tail_link;
	if(t==NULL)
		goto t6;
	
	u=t->head;
	w=t->tail_link;



    t5: if(u==valq)
		/*u=valq:satisfies the constraint,then take next constraint*/

	  goto t1; 
	
	if(w!=NULL)
	{
	  t=w;
	  u=t->head;
	  w=t->tail_link;
	  goto t5;
	}	/*w=NULL:<q,valq> is't present in the head_linkink*/

	/*is there a link from valq to q in order_nodes?*/
    t6: r2=link_value(valq,q);
	if(r2==1)
	  goto t1; 	/* success,take next constraint*/
	if(r2==0){
	  return(0);}  /*there is not the link, fail to satisfy the constraint*/
	if(r2==-1){

	  printf("\n error in check_1 \n");
	  return(-1);}
	
	/*work with tail_linkink of the variable*/	

   tl3: s2=var_node_tail[i].tail_link;



	/* take next constraint_node in the tail_linkink of the variable */
   tl1: if(s2==NULL)	
	   {
	
		iftrace(printf("\n success check1\n"));
	 	return(1);
	   }

	  p2=s2;	/*p2 -> the first node of tail_linkink of the variable*/


	  q2=p2->head;	/*q2 -> the head of a constraint_node in the tail_linkink*/
	  s2=p2->tail_link;	/*s2 -> the next node of the tail_linkink */

	 goto tl211;
	 
	/* take next constraint_node in the tail_linkink of the variable */
 tl211: if(valq!=q2)   
	  goto tl2;	
	goto tl1;	/* valq=q2? if yes,get next constraint in head_linkink */

         		
	/* take the constraints which have't two variables */
	/* work with the head_linkink of order_node_link */
   tl2: for(j=0;j<count_constant;j++)
	{
	 
	 if(q2==domain[j])
	  goto tl4;
	}

	printf("\n error in check_1\t ElemId q2=%ld is not in Domain\n",q2);

	return(-1);

   /*compare q2<=valq with all nodes in the head_linkink of order_node,q2 */
   tl4: t2=dom_node_head[j].head_link;
	if(t2==NULL)
		goto tl6;
	u2=t2->tail;	
	w2=t2->head_link;

   tl5: if(u2==valq)
	  goto tl1;   /*u2=valq:satisfies the constraint,then take next constraint*/
 
	if(w2!=NULL)
	{
	  t2=w2;
	  u2=t2->tail;
	  w2=t2->head_link;
	  goto tl5;
	}	/*w2=NULL:<q2,valq> is't present*/
 tl6:
	/*is there a link from q2 to valq in order_nodes?*/

      r3=link_value(q2,valq);
	if(r3==1)
	  goto tl1; 	/* success,take next constraint*/
	if(r3==0)
	{
	  return(0);
	}  /*there is not the link, fail to satisfy the constraint*/

	  printf("\n error tl6 in check_1 \n");

	  return(-1);
}


/* check_2(struct com_lab *pt): check whether the compound label satisfies all the constraints with two variables,i.e. in array_two_variable_constraint[][]. pt: a pointer to the compound label. success:return 0; fail:return 1; error:return -1.*/

int check_2(struct com_lab *pt)
{
     	
	int h;
	int  t,s;

	iftrace(printf("\n pt[0].value=%d\tpt[1].value=%d\n",pt[0].value,pt[1].value));	



	t=pt[0].value; 		/*t point to the tail*/
	s=pt[1].value;	/*s pointer the head*/	
        h=link_value(domain[s],domain[t]); /* check whether there is a link from s to t*/
	if(h==1)	/*h=1:satisfy, get next constraint_node*/
	{

	iftrace(printf("\n success check2\n"));

	return(1);
	}
	
	if(h==-1){ 	/*h=-1: error*/

	 printf("\n error in check_2\n");


	 return(-1);}

	/*h=0: fail, return to get another compound label*/

	iftrace(printf("\n fail check2\n"));

	 return(0);
}




/* int compatible()
** test whether compound_label+{<x,v>} violates no constraints.
** return:1, yes; return:0, no. 
*/

int compatible(struct two_values *c_label,int num_lab, struct two_values t_label)
{
	int i,j,lab_x,lab_v,k,foot;


	if(num_lab==0)
		return(1);/*violates no constraints*/

	for(i=0;i<num_lab;i++)
	{
		lab_x=c_label[i].value1;
		lab_v=c_label[i].value2;																						
		for(j=0;j<number_binary_variable_constraint;j++)
		{
			if(array_two_variable_constraint[j*2+0]==lab_x && array_two_variable_constraint[j*2+1]==t_label.value1)  /*which constraint?*/
			{
				foot=j;
				goto com11; /* a binary constraint is found*/
			}
			if(array_two_variable_constraint[j*2+0]==t_label.value1 && array_two_variable_constraint[j*2+1]==lab_x)
			{
				foot=j;
				
				goto com12; /* a binary constraint is found*/
			}
		}

		goto com2; /* no constraint between lab_x and z exists, go on  */

	 com11:  for(k=0;k<count_two_variable_constraint[foot];k++)
			if(lab_v==domain_of_two_variable_constraint[foot][k].value1 && t_label.value2==domain_of_two_variable_constraint[foot][k].value2)	
				goto com2; /*compatible with one label, go on test with others*/

		return(0); /*Violates one constraint*/

	 com12:  for(k=0;k<count_two_variable_constraint[foot];k++)
			if(t_label.value2==domain_of_two_variable_constraint[foot][k].value1 && lab_v==domain_of_two_variable_constraint[foot][k].value2)
					goto com2; /*compatible with one label, go on test with others*/

		return(0); /*Violates one constraint*/


	com2:

	}
	return(1);

}


		
/* int *Update_1()
** count_lub_unitll update the doms of the unlabelled variables. 
** It considers binary constraints only.
** return the updated doms.
*/

int *Update_1(int W, int *DD,  struct two_values label,int *idd)
{
	int i,j,v,nonviolate,y,*D2=NULL;
	struct two_values *test_label;

	iftrace(printf("\n enter Update\n"));

   if(W==0)
	return(DD);
   else
   {

        
       test_label =(struct two_values *)calloc(1,sizeof(struct two_values));
       D2=(int*)calloc(count_variable*count_constant,sizeof(int));

	/* D2 <--DD*/
			
	for(i=0;i<count_variable;i++)
	  for(j=0;j<count_constant;j++)
		D2[i*count_constant+j]=DD[i*count_constant+j];
	

	for(i=count_variable-W; i<count_variable;i++)
	{
		
		for(j=idd[i];j<count_constant; j++)
		{
		  	if(D2[i*count_constant+j]!=-1)
		   	{

	   	     	  v=D2[i*count_constant+j];	/*pick one value v from D1y*/
			  y=i;

						  		
	           	  test_label[0].value1=y;
			  test_label[0].value2=v;
			  nonviolate=compatible(test_label,1, label);
			 
			  if(nonviolate==0)
				D2[i*count_constant+j]=-1;
			}
		}
		
	}

	free(test_label);
	test_label=NULL;
	return(D2);

   }
}

						
		

	

/* int *
** foward_check(int *unlabelled, struct two_values *compound_label,int num_label,int *D)
** This function uses an algorithm (FC-1) for Forward Checking in book "Edward Tsang: Foundations of Contraint Satisfaction "
** unlabelled[] : is a set of variables to be labelled;
** compound_label[]: is a set of labels already committed to ;
** D[]:vector[];
** C[]: binary constraints, domain_of_two_variable_constraint[].
*/	

int *
foward_check(int unlabelled, struct two_values *compound_label, int num_label,int *D,int *id)
{
	int i,j,k,x,v,nonviolate, *Result=NULL;
	int i1,j1,i2,k2,k3,k4,*D1=NULL;
	struct two_values temp_label;
	int *nonempty=NULL;
	int *non_finish=NULL;
	int *solving=NULL;

	
 
        nonempty=(int *)calloc(count_variable,sizeof(int));
        non_finish=(int *)calloc(count_variable,sizeof(int));
	 
	if(unlabelled==0)   /*i.e. all variables are labelled.*/
	{
	  solving=(int *)calloc(count_variable,sizeof(int));

	  for(j=0;j<count_variable;j++)
		solving[j]=compound_label[j].value2;   /*solving is a solution*/
	  
	  
	  free(nonempty);
	  free(non_finish);

	  return(solving);
	}

	else
	{
	   
	    i=count_variable-unlabelled;
	    	
	   	for(k=id[i];k<count_constant;k++)
		   if(D[i*count_constant+k]!=-1)
		   {
			 
				num_label=i;
	
			  
	   	     	  v=D[i*count_constant+k];	/*pick one value from Dx*/
		     	  x=i;	 /*pick one variable x from unlabelled*/

		          temp_label.value1=x;
		      	  temp_label.value2=v;

		
			 if((nonviolate=compatible(compound_label,num_label, temp_label))==1)        /*i.e. compound_label+{<x,v>} violates no constraints*/
			  {

				D1=Update_1(unlabelled-1,D,temp_label,id);

		/*update the doms of the unlabelled variables*/

			    /* check if no dom in D1 is empty? */

			 

				for(i1=count_variable-unlabelled+1;i1<count_variable;i1++)
				{
				  nonempty[i1]=0;
				   for(j1=id[i1];j1<count_constant;j1++)
					if(D1[i1*count_constant+j1]!=-1)
					  nonempty[i1]=1;
				}

				for(i2=count_variable-unlabelled+1;i2<count_variable;i2++)
				   if(nonempty[i2]==0)
				   {
				     
				     for(k2=i2;k2>=i+1;k2--)
				     {
					if(k2==i2)
					{
					   for(k3=id[k2]+1;k3<count_constant;k3++)
					     if(D1[k2*count_constant+k3]!=-1)
					     {
						id[k2]++;
						non_finish[k2]=1;
						goto fc4;
					     }
					   non_finish[k2]=0;
					   if(k2==i+1)
						goto fc3;
					   id[k2]=0;
					   goto fc4;
					}
					
					if(non_finish[k2+1]==0)
					{
					  for(k4=id[k2]+1;k4<count_constant;k4++)
					     if(D1[k2*count_constant+k4]!=-1)
					     {
						id[k2]++;
						non_finish[k2]=1;
						goto fc4;
					     }
					  if(k2==i+1)
					  {
					   goto fc3;
					  }
						
					 
					  else
					  {
					   non_finish[k2]=0;
					   id[k2]=0;
					   goto fc4;
					  }
					}
					else /*(non_finish[k2+1]!=0)*/
					{
						non_finish[k2]=1;
						goto fc4;
					}
				     fc4: /*next k2*/
				     }
				   }
					


					compound_label[num_label].value1=temp_label.value1;
					compound_label[num_label].value2=temp_label.value2;


					num_label++;
					
					Result=foward_check(unlabelled-1,compound_label,num_label,D1,id);
					if(Result!=NULL)
					{
					   if(unlabelled<count_variable)
						free(D);
					  if(nonempty != NULL)
					{
					   free(nonempty);
					   nonempty=NULL;
					}
					  if(non_finish !=NULL)
					{

					    free(non_finish);
					   non_finish=NULL;
					}
					   return(Result);
					}

						
				
			  }/*compatible*/

			fc3: 	id[i+1]=0;
				if(D1 !=NULL)
				{
					free(D1);
					D1=NULL;
				}

		    }/*next Dx*/

	       
		if(nonempty != NULL)
		{
		 	free(nonempty);
		 	nonempty=NULL;
		}
		if(non_finish !=NULL)
		{
		free(non_finish);
		non_finish = NULL;
		}
		return(NULL);
		    	
	    
	}/*else*/
}



		
/*int check_constraint(): check whether all compound labels satisfy all the constraints in the solving problem. success: return 1; fail: return 0.*/

int check_constraint()
{      
             
	
	int i,j,i1,j1,f,g,k,j2,i3,j3,f1,pt,ph,f2;
	ElemId pp,c1,c2, sup_lub;
	struct com_lab p[2];
	int bt,bh,kn,k1,total,cfoot,*num_constant_lup=NULL;
	int kp,r1,r2,r3,r5,*dc=NULL;
	int tmp_v,tmp_c;
	int kn1,pc[size4][size2][2],m1, r52;
	struct two_values *comp_label;
	struct two_values up[size4][size1][size2];
	int r21,rg,p2,pc0,r51,r11,r12,Z,*solution=NULL;
	int q,pv1,pv2,pc1;
	int num_comp_label,total_cp0,total_cp1;
	int Ub_number_binary_variable_constraint, Ub_array_two_variable_constraint[size1][2],Ub_count_two_variable_constraint[size1];
	struct two_values Ub_domain_of_two_variable_constraint[size1][size1]; 
	int *selected=NULL;
	int *test_selected=NULL;
	int *nonend=NULL;
	int *flag_variables=NULL;		






	/* to check whether all constraints in form of 
constraint_def_ee(ElemId a, ElemId b):a=< b are fulfilled? */

	for (i=0; i<number_constraint_ee; i++)
	{
		c1=ee1[i];
		c2=ee2[i];
		f=link_value(c1, c2);
		if (f==1)	
		  goto cct1;	/*success, take the next constraint*/
		if (f==0)
		  return(0);	/*fail */
		if(f==-1)
		{
		  printf("\n error 1 in check_constraint\n");
		  return(-1);	/*error */
		}
	  cct1: 

	}

     	vector=(int*)calloc(count_variable*count_constant,sizeof(int));

	
	/*assign the domain of variables as initial value to vectors*/
	
	for(i=0;i<count_variable;i++)
	  for(j=0;j<count_constant;j++)
	     vector[i*count_constant+j]=j;

	if(number_constraint_ev==0 && number_constraint_ve==0)
		goto check_lub;		   

	
		

    /*to check which label<variable,value> satisfies all the constraints in form of constraint_def_ev(ElemId a, CVarId v): a=< v ? */

	for (i=0;i<count_variable;i++)
	{
	  pp=variables[i]; 	/*pp->variable*/
	  for(j=0;j<count_constant;j++)
	  {
	    q=vector[i*count_constant+j];	/*q -> value in solution_vector*/

	    f=check_1(pp,domain[q]);	/*f=1:satisfies(<x,v.,Cx) is true */
	    if(f==-1)		/*f=0:fail to satisfy*/
	    {
		printf("\n error 2 in check_constraint\n");
	     return(-1);	
	    }

	    if(f==0)		/*f=-1:error exit*/ 
	     {

	        vector[i*count_constant+j]=-1;
	     }

	  }

	}


	for(i1=0;i1<count_variable;i1++)
	{
	  k=0;
 	  for(j1=0;j1<count_constant;j1++)
	    if(vector[i1*count_constant+j1]!=-1)
	     {
		k++;

	
	iftrace(printf("vector[%d*%d+%d]=%d\t",i1,count_constant,j1,vector[i1*count_constant+j1]));


	     }


	  if(k==0)
		return(0);	/*fail with constraints "-ev "& "-ve"*/
	} 
		
   check_lub:

	/* check the least upper bound*/


	iftrace(printf("\n number_lub=%d\n",number_lub));


	if(number_lub<=0)
	  goto cck1;	/*simple situation*/

  
	number_variable_lub=(int *)calloc(number_lub, sizeof(int));
	
	dc=(int *)calloc(number_lub, (count_variable+count_constant)*sizeof(int));

	reference_unit_lub=(int *)calloc(number_lub, (count_variable+count_constant)*sizeof(int));

	                            


	/*Analyse all units in upper-bound-constraints */

	kp=-1;/*kp:a count for the number of the units in a upper-bound-constraint*/

	num_constant_lup=(int *)calloc(number_lub, sizeof(int));

	flag_variables=(int *)calloc(number_lub, sizeof(int));

	for(i=0;i<number_lub;i++)
	{/*2*/

	  
	  if(type_array_lub[i*(count_constant+count_variable)+0]==0)	/* ub_first, y is a constant */

	      for(r1=0; r1<count_constant; r1++)
		if (array_lub[i*(count_constant+count_variable)+0]==domain[r1])
		 {
			reference_unit_lub[i*(count_constant+count_variable)+0]=r1;
			goto b2_1;
		 }

	  if(type_array_lub[i*(count_constant+count_variable)+0]==1)   /* ub_first, y, is a variable */
	      for(r11=0;r11<count_variable;r11++)
		if(array_lub[i*(count_constant+count_variable)+0]==variables[r11])
		{
			reference_unit_lub[i*(count_constant+count_variable)+0]=r11; /* wq: a count for the foot number in variables[] of the units in upper-bound-constraints */
			
			goto b2;

		}
/* process with the other parts in upper-bound-constraints*/

       b2_1: for(j=1;j<count_unit_lub[i];j++)	
	    {/*2_1*/

	      if( type_array_lub[i*(count_constant+count_variable)+j]==1) /*It's a variable*/
		for(r12=0;r12<count_variable;r12++)
			if(array_lub[i*(count_constant+count_variable)+j]==variables[r12])
			{
				reference_unit_lub[i*(count_constant+count_variable)+j]=r12;
				flag_variables[i]++;

	/* pc[][0]: a count for the foot number in variables[] of y; pc[][1]:a count for other units in upper-bound-constraints */
			
				goto b1_1;

			}

	      if( type_array_lub[i*(count_constant+count_variable)+j]==0)	/* It's a constant */
		for(r2=0;r2<count_constant;r2++)
			if(array_lub[i*(count_constant+count_variable)+j]==domain[r2])
			{
				reference_unit_lub[i*(count_constant+count_variable)+j]=r2;
				num_constant_lup[i]++;
				goto b1_1;
			}


		b1_1:

	     }/*2_1*/
	   number_variable_lub[i]=0; /* number_: a count for the variable-parts in a upper-bound-constraint*/

	 goto end_b2;



       b2: for(j=1;j<count_unit_lub[i];j++)	
	    {/*1*/

	      if( type_array_lub[i*(count_constant+count_variable)+j]==1) /*It's a variable*/
		for(r12=0;r12<count_variable;r12++)
			if(array_lub[i*(count_constant+count_variable)+j]==variables[r12])
			{
				reference_unit_lub[i*(count_constant+count_variable)+j]=r12;
				
				kp++;

				pc[i][kp][0]=reference_unit_lub[i*(count_constant+count_variable)+0];
	  			pc[i][kp][1]=reference_unit_lub[i*(count_constant+count_variable)+j];

	/* pc[][0]: a count for the foot number in variables[] of y; pc[][1]:a count for other units in upper-bound-constraints */
			
				goto b1;

			}

	      if( type_array_lub[i*(count_constant+count_variable)+j]==0)	/* It's a constant */
		for(r2=0;r2<count_constant;r2++)
			if(array_lub[i*(count_constant+count_variable)+j]==domain[r2])
			{
				reference_unit_lub[i*(count_constant+count_variable)+j]=r2;
				num_constant_lup[i]++;
				goto b1;
			}


		b1:

	     }/*1*/
	   number_variable_lub[i]=kp+1; /* number_: a count for the variable-parts in a upper-bound-constraint*/

	end_b2: kp=-1;
	

	iftrace(printf("\n number_variable_lub[%d]=%d\n",i,number_variable_lub[i]));


	}/*2*/

	
	/*At first, use array_lub[](=vaule) to modify vector[y]*/

	
   
   	for(i=0;i<number_lub;i++)
	{/*i*/

		/*calculate sup_lub(A,B,C,...) */

	   if(number_variable_lub[i]==0 && flag_variables[i]==0)	/*A,B,C,... all are constants*/
	   {

	  	sup_lub=domain[reference_unit_lub[i*(count_constant+count_variable)+1]];

	
	  	for(m1=2;m1<count_unit_lub[i];m1++)
		{

		    		sup_lub=lub(sup_lub,domain[reference_unit_lub[i*(count_constant+count_variable)+m1]]);
		}
		
	 	if(type_array_lub[i*(count_constant+count_variable)+0]==0)	/*y is a constant */
	 	   if(array_lub[i*(count_constant+count_variable)+0]!= sup_lub)
		   {
			if(reference_unit_lub !=NULL)
			{
			free(reference_unit_lub);
			reference_unit_lub=NULL;
			}
			if(num_constant_lup != NULL)
			{
			free(num_constant_lup);
			num_constant_lup=NULL;
			}
			if(dc !=NULL)
			{
			free(dc);
			dc=NULL;
			}
			if(number_variable_lub !=NULL)
			{
			free(number_variable_lub);
			number_variable_lub=NULL;
			}

			return(0);   /*not satisfiable*/
		  }

		
		if(type_array_lub[i*(count_constant+count_variable)+0]==1)	/* y is a variable*/
		{
			for(r51=0;r51<count_constant;r51++)
			 if(vector[reference_unit_lub[i*(count_constant+count_variable)+0]*count_constant+r51] != -1)
			   if(domain[vector[reference_unit_lub[i*(count_constant+count_variable)+0]*count_constant+r51]] ==sup_lub)
			   	goto fix;

			if(reference_unit_lub !=NULL)
			{
			free(reference_unit_lub);
			reference_unit_lub=NULL;
			}
			if(num_constant_lup != NULL)
			{
			free(num_constant_lup);
			num_constant_lup=NULL;
			}
			if(dc !=NULL)
			{
			free(dc);
			dc=NULL;
			}
			if(number_variable_lub !=NULL)
			{
			free(number_variable_lub);
			number_variable_lub=NULL;
			}

			return(0);  /*not satisfiable*/

		fix:	for(r52=0;r52<count_constant;r52++)
			  if(r52!=r51)
			   vector[reference_unit_lub[i*(count_constant+count_variable)+0]*count_constant+r52]=-1;
		}
	  }

	else
	{/*number_variable_lub[]!=0 or flag_variables[i]!=0*/

	  if(type_array_lub[i*(count_constant+count_variable)+0]==0)	/*y is a constant */
	  {/*b41*/

	   	 r21=1;
	    b41: while(r21<count_unit_lub[i])
	    	{ /*r21*/

		   rg=0;
		   if(type_array_lub[i*(count_constant+count_variable)+r21]==0)	/* wt=0: constant*/

			{
				p2=reference_unit_lub[i*(count_constant+count_variable)+r21];
				pc0=reference_unit_lub[i*(count_constant+count_variable)+0];
				f2=link_value(domain[p2],domain[pc0]); /*p2<=pc0 ? */

				if(f2==-1)
				{
					printf("\n error 3 in check_constraint\n");

					return(-1);
				}

				if(f2==0)
					return(0); /*fail*/
				if(f2==1)
				{
					r21++;
					goto b41; /* go on the next part*/
				}

			}							   					

		   if(type_array_lub[i*(count_constant+count_variable)+r21]==1)	/* wt=1: variable*/
			for(r51=0;r51<count_constant;r51++)
			  if(vector[reference_unit_lub[i*(count_constant+count_variable)+r21]*count_constant+r51]!=-1)
			  {
				p2=vector[reference_unit_lub[i*(count_constant+count_variable)+r21]*count_constant+r51];
				pc0=reference_unit_lub[i*(count_constant+count_variable)+0];
				f2=link_value(domain[p2],domain[pc0]); /*p2<=pc0 ? */
		  	  	if(f2==-1)
				{
					printf("\n error 4 in check_constraint\n");

					return(-1);
				}
				if(f2==1)
				{
				   if(num_constant_lup[i]==1)
				   
				     for(cfoot=1; cfoot<count_unit_lub[i];cfoot++)
					if(type_array_lub[i*(count_constant+count_variable)+cfoot]==0)
					  if(p2==reference_unit_lub[i*(count_constant+count_variable)+cfoot])
					   {
						vector[reference_unit_lub[i*(count_constant+count_variable)+r21]*count_constant+r51]=-1;
						if(r51==(count_constant-1))
					      if(rg==0)
						return(0);/* fail*/
					  }
				  rg=1;
				}

				if(f2==0)
				{
					vector[reference_unit_lub[i*(count_constant+count_variable)+r21]*count_constant+r51]=-1;

					if(r51==(count_constant-1))
					   if(rg==0)
						return(0);/* fail*/
				}
						
		     	  } /*take the next value of vector[]*/

		    r21++;
		    goto b41; /* go on the next part*/
 			
	    	}/*r21*/


	  	goto bi;
			
	  }/*b41*/


	  else 		/*type_array_lub[i][0]==1: y is a variable*/
	  { /*b4*/

	   	r2=1;
	    b4: while(r2<count_unit_lub[i])
	    	{

	iftrace(printf("\n count_unit_lub[%d]=%d\t r2=%d\n",i,count_unit_lub[i],r2));

		
			if(type_array_lub[i*(count_constant+count_variable)+r2]==0)	/* wt=0: constant*/

			{
				pc1=reference_unit_lub[i*(count_constant+count_variable)+r2];
				r2++;
				goto b3;
			}

		  	r2++;

	    	}

	  	goto bi;
	
	   b3: for (r5=0;r5<count_constant;r5++)
			if(vector[reference_unit_lub[i*(count_constant+count_variable)+0]*count_constant+r5]!=-1)
			{

				pv1=vector[reference_unit_lub[i*(count_constant+count_variable)+0]*count_constant+r5];
				

	iftrace(printf("\n pc1=%lu\tpv1=%lu\n",pc1,domain[pv1]));

				f1=link_value(domain[pc1],domain[pv1]); /*pc1<=pv1 ? */

				if(f1==-1)
				{
				printf("\n error 5 in check_constraint\n");

					return(-1);
				}

				if(f1==0)
				   vector[reference_unit_lub[i*(count_constant+count_variable)+0]*count_constant+r5]=-1;

				

			}
		
		goto b4;
	
	  }/*b4*/

	 bi:
	}/*number_variable_lub[]!=0*/

	}/*i*/
	/* already finished to process all the parts if y is a constant and finished to process all constant-parts if y is a variable */




		/*then, use array_lub[](=variable) to modify vector[y]*/


    	kn1=0; /*kn1: a temperary count for the different values of every up[][]*/




	for(i=0;i<number_lub;i++)
	{/*b5-i*/

	  dc[i*(count_constant+count_variable)+0]=0;

	  if(type_array_lub[i*(count_constant+count_variable)+0]==0)
		goto b5i;

	  else	/* wt=1*/
	  
           for(j2=0;j2<number_variable_lub[i];j2++)
	   {/*b5-j2*/
   
		
	   	pt=pc[i][j2][0];	/*get a binary constraint*/
	   	ph=pc[i][j2][1];	
	   	   
	   	for(j=0;j<count_constant;j++)
	   	 if(vector[pt*count_constant+j]!=-1)
	     	 {/*3*/
	  
	  	   pv1=vector[pt*count_constant+j]; /*p[0]:pointer to <i,b> of C<i,j>*/	
                   total=0;
		   p[1].num=0;
		   for(k=0;k<count_constant;k++)
	            if(vector[ph*count_constant+k]!=-1)
		     {/*k*/
  
	  	    	pv2=vector[ph*count_constant+k]; 
					/*p[1]: pointer to <j,c> of C<i,j>*/

	iftrace(printf("\npv2=%lu\tpv1=%lu\n",domain[pv2],domain[pv1]));

		   	
    	                g=link_value(domain[pv2],domain[pv1]);	/*check the compound label pv2<=pv1?*/  

		        if(g==-1)	/*g=-1:error*/
			{
			printf("\n error 6 in check_constraint\n");

	   	          return(-1);
			}

		        if(g==1) 
		 		/* g=1:satisfy,get the next compound label */

		         {
			  if(num_constant_lup[i]==1 && count_unit_lub[i]==3)
				for(m1=1;m1<count_unit_lub[i];m1++)
				   if(type_array_lub[i*(count_constant+count_variable)+m1]==0)
					if(reference_unit_lub[i*(count_constant+count_variable)+m1]==pv2)
					{
					  total++;
					  up[i][j2][kn1].value1=pv2;
					  up[i][j2][kn1].value2=pv2;
					  kn1++;
					  goto next_k;
					}
					
					
			  
			  					
	 	    	   total++;

	 	    	   up[i][j2][kn1].value1=pv1;  /*find a 2-compound label <(i,b><j,c>)*/
	 	           up[i][j2][kn1].value2=pv2;
			  
	 	           

		

			  kn1++;
		         }
		next_k:		/*g=0:fail, get next <j,c>*/

		     }/*k*/

		   if(total==0)
			vector[pt*count_constant+j]=-1;

	     	  }/*3*/

	  	dc[i*(count_constant+count_variable)+j2]=kn1; /*dc: a count for the diffrent values of every up[][]*/

	  	kn1=0;

	iftrace(printf("\ndc[%d][%d]=%d\n",i,j2,dc[i*(count_constant+count_variable)+j2]));

	 }/*b5-j2*/ 


     b5i:   

	}/*b5-i*/



	/* change up[] into Ub_domain_of_two_variable_constraint[] */

	Ub_number_binary_variable_constraint=0;

	for(i=0;i<number_lub;i++)
	  for(j=0;j<number_variable_lub[i];j++)
	  {
		Ub_count_two_variable_constraint[Ub_number_binary_variable_constraint]=dc[i*(count_constant+count_variable)+j];
		Ub_array_two_variable_constraint[Ub_number_binary_variable_constraint][0]=pc[i][j][0];
		Ub_array_two_variable_constraint[Ub_number_binary_variable_constraint][1]=pc[i][j][1];
		for(k=0;k<dc[i*(count_constant+count_variable)+j];k++)
		{
		   	
			Ub_domain_of_two_variable_constraint[Ub_number_binary_variable_constraint][k].value1=up[i][j][k].value1;
			Ub_domain_of_two_variable_constraint[Ub_number_binary_variable_constraint][k].value2=up[i][j][k].value2;
			
		}
		Ub_number_binary_variable_constraint++;
	  }


	/*creat the least upper bound*/

 	/* check : is there  only one upper-bound-value  ? */

	   
	
   cck1:
						

	
	solutions=(int *)calloc(size_solution*count_variable,sizeof(int));

	
	  for(j=0;j<count_variable;j++)
		solutions[0+j]=-1;

	


	if(number_binary_variable_constraint==0)	/*if there is no binary constraint*/
	{

	    if(number_lub>=1)
	    {
	      if(Ub_number_binary_variable_constraint>0)
	      {
		array_two_variable_constraint=(int *)srealloc((char *)array_two_variable_constraint,Ub_number_binary_variable_constraint*2*sizeof(int));
		count_two_variable_constraint=(int *)srealloc((char *)count_two_variable_constraint,Ub_number_binary_variable_constraint*sizeof(int));
	      }
   		  for(i1=0;i1<number_lub;i1++)
			if(number_variable_lub[i1]!=0)
			{
				/* change Ub_domain_of_two_variable_constraint[] to domain_of_two_variable_constraint[], etc.*/
				number_binary_variable_constraint=Ub_number_binary_variable_constraint;
				for(j=0;j<number_binary_variable_constraint;j++)
				{
					array_two_variable_constraint[j*2+0]=Ub_array_two_variable_constraint[j][0];
					array_two_variable_constraint[j*2+1]=Ub_array_two_variable_constraint[j][1];
					count_two_variable_constraint[j]=Ub_count_two_variable_constraint[j];
					for(k=0;k<Ub_count_two_variable_constraint[j];k++)
					{
						domain_of_two_variable_constraint[j][k].value1=Ub_domain_of_two_variable_constraint[j][k].value1;
						domain_of_two_variable_constraint[j][k].value2=Ub_domain_of_two_variable_constraint[j][k].value2;
					}
				}

		
				goto seeksolution;
			}

		  goto d0;  /*number_binary_variable_constraint=0 and number_variable_lub[]=0*/


	    }


		/* number_binary_variable_constraint=0 and number_lub=0*/
			
	   d0:  
		j3=0;
	 	 for(i3=0;i3<count_constant;i3++)
	      	   if(vector[0*count_constant+i3]!=-1)
			{	
		 	 solutions[j3*count_variable+0]=vector[0*count_constant+i3];
			 j3++;
			}
	   
	  	count_solution=j3;

		if(count_solution>size_solution)
		{
			size_solution=count_solution;
			solutions=(int *)srealloc((char *)solutions,sizeof(int)*size_solution*count_variable);
	
		}

		for(j1=1;j1<count_variable;j1++)
	
		  for(r1=0;r1<count_constant;r1++)

		    if(vector[j1*count_constant+r1]!=-1)
	  
		       if(solutions[0*count_variable+j1]==-1)

			 for(k1=0;k1<count_solution;k1++)
			    solutions[k1*count_variable+j1]=vector[j1*count_constant+r1];

		       else 	/*solutions[0][j1]!=-1*/
		       {

		if(count_solution*2>size_solution)
		{
			size_solution=count_solution*2;
			solutions=(int *)srealloc((char *)solutions,sizeof(int)*size_solution*count_variable);
					}



			 for(k1=0;k1<count_solution;k1++)
			 {

			    for(j2=0;j2<j1;j2++)
				solutions[(count_solution+k1)*count_variable+j2]=solutions[k1*count_variable+
j2];

			    solutions[(count_solution+k1)*count_variable+j1]=vector[j1*count_constant+r1];

			 }
		  
		  	 count_solution=count_solution*2;

		       }
	iftrace(printf("\n B0 count_solution=%d\n",count_solution));	

	  	goto st2;	/*output*/

	}/*if(number_binary_variable_constraint==0)*/


	/*number_binary_variable_constraint!=0 : with constaints of constraint_def_vv, construct domain_of_two_variable_constraint[]*/


	kn=0;
      for(i=0;i<number_binary_variable_constraint;i++)
	{
  
	   bt=array_two_variable_constraint[i*2+0];	/*get a binary consrtaint*/
	   bh=array_two_variable_constraint[i*2+1];	
	   p[0].num=0; 
  
	   for(j=p[0].num;j<count_constant;j++)
	    if(vector[bt*count_constant+j]!=-1)
	     {
	  
	  	   p[0].value=vector[bt*count_constant+j]; /*p[0]:pointer to <i,b> of C<i,j>*/
	           p[0].num=j+1;


	iftrace(printf("\n bt=%d\t vector[%d*%d+%d]=%d\n",bt,bt,count_constant,j,domain[vector[bt*count_constant+j]]));
	
                  total=0;
		   p[1].num=0;
		   for(k=p[1].num;k<count_constant;k++)
	            if(vector[bh*count_constant+k]!=-1)
		     {


	iftrace(printf("\nbh=%d\t vector[%d*%d+%d]=%d\n",bh,bh,count_constant,k,domain[vector[bh*count_constant+k]]));   
	
	  
	  	    	p[1].value=vector[bh*count_constant+k]; 
					/*p[1]: pointer to <j,c> of C<i,j>*/
	           	p[1].num=k+1;
		   	
    	               g=check_2(p);	/*check the compound label*/  
		        if(g==-1)	/*g=-1:error*/
			{
			  printf("\n error 11 in check_constraint\n");
	   	          return(-1);
			}
		        if(g==1) 
		 		/* g=1:satisfy,get the next compound label */
		         {
	 	    	   total++;
	 	    	   domain_of_two_variable_constraint[i][kn].value1=p[0].value;  /*find a 2-compound label <(i,b><j,c>)*/
	 	           domain_of_two_variable_constraint[i][kn].value2=p[1].value;
	 	           kn++;
		         }
				/*g=1:fail, get next <j,c>*/
		     }
		   if(total==0)
			vector[bt*count_constant+j]=-1;
	     }
	  count_two_variable_constraint[i]=kn;
	  kn=0;

	iftrace(printf("\n count_two_variable_constraint[%d]=%d \n",i,count_two_variable_constraint[i]));
	  

	 }/*for(i=0;i<number_binary_variable_constraint;i++)*/

	/* To find out the maximum of count_two_variable_constraint[] */

/*	max=count_two_variable_constraint[0]; 
	for(i=1;i<number_binary_variable_constraint;i++)
		if(max<=count_two_variable_constraint[i])
			max=count_two_variable_constraint[i]; 

	cp=(struct two_values *)srealloc((char *)cp,ncb*max*sizeof(struct two_values));*/
		


	if(number_lub>=1)
	{
	  array_two_variable_constraint=(int *)srealloc((char *)array_two_variable_constraint, 2*(number_binary_variable_constraint+Ub_number_binary_variable_constraint)*sizeof(int));

	  count_two_variable_constraint=(int *)srealloc((char *)count_two_variable_constraint,(number_binary_variable_constraint+Ub_number_binary_variable_constraint)*sizeof(int));

		/* add Ub_domain_of_two_variable_constraint[] to domain_of_two_variable_constraint[], then goto seeksolution*/

		for(j=number_binary_variable_constraint;j<number_binary_variable_constraint+Ub_number_binary_variable_constraint;j++)
		{
			array_two_variable_constraint[j*2+0]=Ub_array_two_variable_constraint[j-number_binary_variable_constraint][0];
			array_two_variable_constraint[j*2+1]=Ub_array_two_variable_constraint[j-number_binary_variable_constraint][1];
			count_two_variable_constraint[j]=Ub_count_two_variable_constraint[j-number_binary_variable_constraint];
			for(k=0;k<Ub_count_two_variable_constraint[j-number_binary_variable_constraint];k++)
			{
				domain_of_two_variable_constraint[j][k].value1=Ub_domain_of_two_variable_constraint[j-number_binary_variable_constraint][k].value1;
				domain_of_two_variable_constraint[j][k].value2=Ub_domain_of_two_variable_constraint[j-number_binary_variable_constraint][k].value2;
			}
		}
		number_binary_variable_constraint=number_binary_variable_constraint+Ub_number_binary_variable_constraint;


		goto seeksolution;		
	}
	

	if(number_binary_variable_constraint==1)	/*if there is only a binary constraint*/
	{


	 	   for(i=0;i<count_two_variable_constraint[0];i++)
	   	   {
			solutions[i*count_variable+array_two_variable_constraint[0*2+0]]=domain_of_two_variable_constraint[0][i].value1;
			solutions[i*count_variable+array_two_variable_constraint[0*2+1]]=domain_of_two_variable_constraint[0][i].value2;
	    	   }
	 	   count_solution=count_two_variable_constraint[0];

	iftrace(printf("\n B1 count_solution=%d\t  number_binary_variable_constraint=%d",count_solution,number_binary_variable_constraint));
		
	}
	/* number_binary_variable_constraint!=1, call fc_1 with domain_of_two_variable_constraint[],number_binary_variable_constraint to seeksolution*/

	


 seeksolution:

	/*Construct the solution*/

	/* forward_checking-1 */




	/* pre-process of vector[] with domain_of_two_variable_constraint[]*/

     	
	  for(j=0;j<number_binary_variable_constraint;j++)
	     for(i1=0;i1<count_constant;i1++)
	     {
		total_cp0=0;
		total_cp1=0;
		for(k=0;k<count_two_variable_constraint[j];k++)
		{
			if(vector[array_two_variable_constraint[j*2+0]*count_constant+i1]==domain_of_two_variable_constraint[j][k].value1)
				total_cp0++;
			if(vector[array_two_variable_constraint[j*2+1]*count_constant+i1]==domain_of_two_variable_constraint[j][k].value2)
				total_cp1++;
		}
		if(total_cp0==0)
			vector[array_two_variable_constraint[j*2+0]*count_constant+i1]=-1;
		if(total_cp1==0)
			vector[array_two_variable_constraint[j*2+1]*count_constant+i1]=-1;
	    }


	/*pre-process of domain_of_two_variable_constraint[] with vector[]*/

	for(i=0;i<number_binary_variable_constraint;i++)
	   for(j=0;j<count_two_variable_constraint[i];j++)
	   {
		total_cp0=0;
		total_cp1=0;

		for(k=0;k<count_constant;k++)
		{
		  if(domain_of_two_variable_constraint[i][j].value1==vector[array_two_variable_constraint[i*2+0]*count_constant+k])
			total_cp0++;
		  if(domain_of_two_variable_constraint[i][j].value2==vector[array_two_variable_constraint[i*2+1]*count_constant+k])
			total_cp1++;
		}
		if(total_cp0==0 || total_cp1==0 )
		{
			domain_of_two_variable_constraint[i][j].value1=-1;
			domain_of_two_variable_constraint[i][j].value2=-1;
		}

	  }



	/* Second pre-process of vector[] with domain_of_two_variable_constraint[]*/

     	
	  for(j=0;j<number_binary_variable_constraint;j++)
	     for(i1=0;i1<count_constant;i1++)
	     {
		total_cp0=0;
		total_cp1=0;
		for(k=0;k<count_two_variable_constraint[j];k++)
		{
			if(vector[array_two_variable_constraint[j*2+0]*count_constant+i1]==domain_of_two_variable_constraint[j][k].value1)
				total_cp0++;
			if(vector[array_two_variable_constraint[j*2+1]*count_constant+i1]==domain_of_two_variable_constraint[j][k].value2)
				total_cp1++;
		}
		if(total_cp0==0)
			vector[array_two_variable_constraint[j*2+0]*count_constant+i1]=-1;
		if(total_cp1==0)
			vector[array_two_variable_constraint[j*2+1]*count_constant+i1]=-1;
	    }

		

	selected=(int *)calloc(count_variable,sizeof(int));
	test_selected=(int *)calloc(count_variable,sizeof(int));
	nonend=(int *)calloc(count_variable,sizeof(int));

	comp_label=(struct two_values *)calloc(count_variable,sizeof(struct two_values));

	r3=0;
	for(i=0;i<count_variable;i++)
		selected[i]=0;
	
search:
	for(j2=0;j2<count_variable;j2++)
	test_selected[j2]=selected[j2];
	

	comp_label[0].value1=-1;
	Z=count_variable;
	num_comp_label=0;

	iftrace(printf("\n search\n\n"));
	solution=foward_check(Z, comp_label,num_comp_label,vector,test_selected);
	if(solution!=NULL)
	{

	/*check if the searched solution is right, with least-upper-bound*/
	 for(i=0;i<number_lub;i++)
   	  if(number_variable_lub[i]>=1)
	  {
	     tmp_v=-1;
	     tmp_c=-1;

	     for(j=1;j<count_unit_lub[i];j++)
	     {
		if(type_array_lub[i*(count_constant+count_variable)+j]==1)
		  if(tmp_v==-1)
		    tmp_v=solution[reference_unit_lub[i*(count_constant+count_variable)+j]];
		  else
		    if(tmp_v!=solution[reference_unit_lub[i*(count_constant+count_variable)+j]])
			goto next_i;
		    
		if(type_array_lub[i*(count_constant+count_variable)+j]==0)
		 if(tmp_c==-1) 
		   tmp_c=reference_unit_lub[i*(count_constant+count_variable)+j];
		 else
		   goto next_i;
	     }

	     if(tmp_v!=tmp_c)
		goto next_i;
	     else
		if(solution[reference_unit_lub[i*(count_constant+count_variable)+0]]!=tmp_c)
			goto next_search;
	  next_i:
	 } /*the searched solution is right*/



			iftrace(printf("\n success \n"));
		for(r1=0;r1<count_variable;r1++) /*success to find a solving*/
		{
			solutions[r3*count_variable+r1]=solution[r1];

		}

		r3++;
	/*	printf("\n r3=%d\n",r3);*/

	
		free(solution);
	

		count_solution=r3;
		goto end_combin;
		
	}

	else	/* no solution*/
	
			count_solution=0;  /* this CPS is not satisfied, goto output*/
			goto end_combin;
	 	
		
		
  next_search:
	  /* create selected[count_variable]*/

           for(i=0;i<count_variable;i++)
		for(j=0;j<count_constant;j++)
			if(solution[i]==vector[i*count_constant+j])
				selected[i]=j;

	        free(solution);
		solution=NULL;
		
		
	/* selected[] is changed*/

        for(i=count_variable-1;i>=0;i--)
	   {
	   	if(i==count_variable-1)
	   	{
		   for(j=selected[i]+1;j<count_constant;j++)
			if(vector[i*count_constant+j]!=-1)
			{
				nonend[i]=1;
				selected[i]++;
				goto se1; /*not changed */
			}
		   nonend[i]=0;
		   for(k=0;k<count_constant;k++)
		   	if(vector[i*count_constant+k]!=-1)
			{
		   		selected[i]=k;
		   		goto se1;
			}
	   	}

	   	if(nonend[i+1]==0)
		{
		    for(j1=selected[i]+1;j1<count_constant;j1++)
			if(vector[i*count_constant+j1]!=-1)
			{
				nonend[i]=1;
				selected[i]=j1;
				goto se1; 				
			}
		    if(i==0)
			goto endstop;

		    else
		    {
		    
		    
		    	nonend[i]=0;
		   	for(k1=0;k1<count_constant;k1++)
		   		if(vector[i*count_constant+k1]!=-1)
				{
				    	selected[i]=k1;
					goto se1;

				}
		    }
		    
						
		}

		else  /*nonend[i+1]=1*/

		{
			nonend[i]=1;
			goto se1; /*selected[i] is not changed*/
		}
	
	   se1:

	  }
		/* To search next solution*/
    



	goto search;
	

        endstop: count_solution=r3;
	

	

	end_combin:    if(selected !=NULL)
			{
			 free(selected);
			 selected=NULL;
			}
			if(nonend !=NULL)
			{
			free(nonend);
			nonend=NULL;
			}
			if(test_selected !=NULL)
			{
			free(test_selected);
			test_selected=NULL;
			}
			if(comp_label !=NULL)
			{
			free(comp_label);
			comp_label=NULL;
			}

		



		

	/* 	enf of connecting for number_binary_variable_constraint */



	
  st2:	/* check_constraints(): success!*/

	
			if(num_constant_lup != NULL)
			{
			free(num_constant_lup);
			num_constant_lup=NULL;
			}
			if(dc !=NULL)
			{
			free(dc);
			dc=NULL;
			}
			if(flag_variables !=NULL)
			{
				free(flag_variables );
				flag_variables =NULL;
			}


	if(count_solution==0)
		return(0);

	return(1);


 }



/*
**int search_all(): To search for all solutions;
** return:0, no solution; return:1, success, 
**the solution is stored in array solutions[], count_solution: the number of solution.
*/

int search_all()
{
	int *selected=NULL;
	int *test_selected=NULL;
	int *nonend=NULL;
	int num_comp_label,tmp_v,tmp_c;
	int Z,r3,i,j,r1,k,j2,j1,k1,*solution=NULL;
	struct two_values *comp_label=NULL;

	
	if(number_binary_variable_constraint==0)
		return(1);		

	selected=(int *)calloc(count_variable,sizeof(int));
	test_selected=(int *)calloc(count_variable,sizeof(int));
	nonend=(int *)calloc(count_variable,sizeof(int));

	comp_label=(struct two_values *)calloc(count_variable,sizeof(struct two_values));

	r3=0;
	for(i=0;i<count_variable;i++)
		selected[i]=0;
	

search: for(j2=0;j2<count_variable;j2++)
		test_selected[j2]=selected[j2];
	

	comp_label[0].value1=-1;
	Z=count_variable;
	num_comp_label=0;

	iftrace(printf("\n search\n\n"));

	solution=foward_check(Z, comp_label,num_comp_label,vector,test_selected);
	if(solution!=NULL)
	{

	/*check if the searched solution is right, with least-upper-bound*/
	 for(i=0;i<number_lub;i++)
   	  if(number_variable_lub[i]>=1)
	  {
	     tmp_v=-1;
	     tmp_c=-1;

	     for(j=1;j<count_unit_lub[i];j++)
	     {
		if(type_array_lub[i*(count_constant+count_variable)+j]==1)
		  if(tmp_v==-1)
		    tmp_v=solution[reference_unit_lub[i*(count_constant+count_variable)+j]];
		  else
		    if(tmp_v!=solution[reference_unit_lub[i*(count_constant+count_variable)+j]])
			goto next_i;
		    
		if(type_array_lub[i*(count_constant+count_variable)+j]==0)
		 if(tmp_c==-1) 
		   tmp_c=reference_unit_lub[i*(count_constant+count_variable)+j];
		 else
		   goto next_i;
	     }

	     if(tmp_v!=tmp_c)
		goto next_i;
	     else
		if(solution[reference_unit_lub[i*(count_constant+count_variable)+0]]!=tmp_c)
			goto next_search;
	  next_i:
	 } /*the searched solution is right*/

			iftrace(printf("\n success \n"));

		for(r1=0;r1<count_variable;r1++) /*success to find a solving*/
		{
		  if(r3 >size_solution-1)
		   {
			size_solution=size_solution+size3;
			solutions=(int *)srealloc((char *)solutions,sizeof(int)*size_solution*count_variable);
			if(solutions==NULL)
				return(-1);
		   }
			solutions[r3*count_variable+r1]=solution[r1];
			iftrace(printf("solution[%d]=%d\t",r1,solution[r1]));
		}
	
		r3++;
		iftrace(printf("\n r3=%d\n",r3));



next_search:
	  /* create selected[count_variable]*/

           for(i=0;i<count_variable;i++)
		for(j=0;j<count_constant;j++)
			if(solution[i]==vector[i*count_constant+j])
				selected[i]=j;

	        free(solution);
		solution=NULL;
		goto change;
		
	}

	else	/* no solution*/
	
		if(r3==0)
		{
			count_solution=0;  /* this CPS is not satisfied, goto output*/
			goto end_combin;
		}
		else	/*r3!=0*/
		{
			iftrace(printf("\n fail \n"));
			goto change; /*go on next combination*/
		}
	
	/* selected[] is changed*/

  change: 
 	   for(i=count_variable-1;i>=0;i--)
	   {
	   	if(i==count_variable-1)
	   	{
		   for(j=selected[i]+1;j<count_constant;j++)
			if(vector[i*count_constant+j]!=-1)
			{
				nonend[i]=1;
				selected[i]++;
				goto se1; /*not changed */
			}
		   nonend[i]=0;
		   for(k=0;k<count_constant;k++)
		   	if(vector[i*count_constant+k]!=-1)
			{
		   		selected[i]=k;
		   		goto se1;
			}
	   	}

	   	if(nonend[i+1]==0)
		{
		    for(j1=selected[i]+1;j1<count_constant;j1++)
			if(vector[i*count_constant+j1]!=-1)
			{
				nonend[i]=1;
				selected[i]=j1;
				goto se1; 				
			}
		    if(i==0)
			goto endstop;

		    else
		    {
		    
		    
		    	nonend[i]=0;
		   	for(k1=0;k1<count_constant;k1++)
		   		if(vector[i*count_constant+k1]!=-1)
				{
				    	selected[i]=k1;
					goto se1;

				}
		    }
		    
						
		}

		else  /*nonend[i+1]=1*/

		{
			nonend[i]=1;
			goto se1; /*selected[i] is not changed*/
		}
	
	   se1:

	  }
		/* To search next solution*/
    



	goto search;
	

        endstop: count_solution=r3;
	

	end_combin:   if(selected !=NULL)
			{
			 free(selected);
			 selected=NULL;
			}
			if(nonend !=NULL)
			{
			free(nonend);
			nonend=NULL;
			}
			if(test_selected !=NULL)
			{
			free(test_selected);
			test_selected=NULL;
			}
			if(comp_label !=NULL)
			{
			free(comp_label);
			comp_label=NULL;
			}
			if(number_variable_lub !=NULL)
			{
			free(number_variable_lub);
			number_variable_lub=NULL;
			}
			if(reference_unit_lub !=NULL)
			{
			free(reference_unit_lub);
			reference_unit_lub=NULL;
			}


	/* 	enf of connecting for number_binary_variable_constraint */

	
  	/* check_constraints(): success!*/


	if(count_solution==0)
		return(0);

	return(1);


 }
	  
int solvable_first(void)
/* return:  1    constraints loesbar */
/*          0    constraints nicht loesbar */
{
	int flag;

#ifdef PRINT 
  printf("solvable_first();\n"); 
#endif	


	
	constraint_graph();

	flag=check_constraint();

	if(flag==-1)
	{
		printf("\n ERROR! \n");
	
		return(-1);	/*error*/
	}

	if(flag==0)
		return(0);	/*fail*/
	else /*(flag==1)*/
		return(1);	/*success*/


}


int solvable_all(void)
/* return:  1    constraints loesbar with all solutions*/
/*          0    constraints nicht loesbar */
{
	int flag;
	
#ifdef PRINT 
  printf("solvable_all();\n"); 
#endif

	flag=search_all();

	if (flag==1)
	  return(1);	/*success*/
	else  /*(flag==0)*/
	  return(0);	/*fail*/


}

void reset_constraints(void)
/* alle Constraints loeschen, damit man danach neue
   Constraints definieren kann
*/
{
#ifdef PRINT 
  printf("reset_constraints();\n"); 
#endif
  if (variables != NULL)
	{
		free(variables);
		variables=NULL;
	}

  if (ee1 != NULL)
	{
		free(ee1);
		ee1=NULL;
	}
  if (ee2 != NULL)
	{
		free(ee2);
		ee2=NULL;
	}
  if (vv1 != NULL)
	{
		free(vv1);
		vv1=NULL;
	}
  if (vv2 != NULL)
	{
		free(vv2);
		vv2=NULL;
	}
  if (ve1 != NULL)
	{
		free(ve1);
		ve1=NULL;
	}
  if (ve2 != NULL)
	{
		free(ve2);
		ve2=NULL;
	}
  if (ev2 != NULL)
	{
		free(ev2);
		ev2=NULL;
	}

  if (ev1 != NULL)
	{
		free(ev1);
		ev1=NULL;
	}

  if (count_unit_lub != NULL)
	{
		free(count_unit_lub);
		count_unit_lub=NULL;
	}
  if (array_lub != NULL)
	{
		free(array_lub);
		array_lub=NULL;
	}
  if (type_array_lub != NULL)
	{
		free(type_array_lub);
		type_array_lub=NULL;
	}
  if (array_two_variable_constraint != NULL)
	{
		free(array_two_variable_constraint);
		array_two_variable_constraint=NULL;
	}
  if (count_two_variable_constraint != NULL)
	{
		free(count_two_variable_constraint);
		count_two_variable_constraint=NULL;
	}
  if (solutions != NULL)
	{
		free(solutions);
		solutions=NULL;
	}
  if (constraint_node != NULL)
	{
		free(constraint_node);
		constraint_node=NULL;
	}

  if (var_node_head != NULL)
	{
		free(var_node_head);
		var_node_head=NULL;
	}
  if (var_node_tail != NULL)
	{
		free(var_node_tail);
		var_node_tail=NULL;
	}
  if (vector != NULL)
	{
		free(vector);
		vector=NULL;
	}




	
	count_variable=0;
	number_lub=0;
	number_constraint_ee=0;
	number_constraint_ev=0;
	number_constraint_ve=0;
	number_constraint_vv=0;
	count_lub_constraint=0;
	count_lub_unit=2;

	count_solution=0;

	

}

void free_space(void)
/* To release all the memory allocated*/
{
#ifdef PRINT 
  printf("free_space();\n"); 
#endif
  if (variables != NULL)
	{
		free(variables);
		variables=NULL;
	}

  if (ee1 != NULL)
	{
		free(ee1);
		ee1=NULL;
	}
  if (ee2 != NULL)
	{
		free(ee2);
		ee2=NULL;
	}
  if (vv1 != NULL)
	{
		free(vv1);
		vv1=NULL;
	}
  if (vv2 != NULL)
	{
		free(vv2);
		vv2=NULL;
	}
  if (ve1 != NULL)
	{
		free(ve1);
		ve1=NULL;
	}
  if (ve2 != NULL)
	{
		free(ve2);
		ve2=NULL;
	}
  if (ev2 != NULL)
	{
		free(ev2);
		ev2=NULL;
	}

  if (ev1 != NULL)
	{
		free(ev1);
		ev1=NULL;
	}

  if (count_unit_lub != NULL)
	{
		free(count_unit_lub);
		count_unit_lub=NULL;
	}
  if (array_lub != NULL)
	{
		free(array_lub);
		array_lub=NULL;
	}
  if (type_array_lub != NULL)
	{
		free(type_array_lub);
		type_array_lub=NULL;
	}
  if (array_two_variable_constraint != NULL)
	{
		free(array_two_variable_constraint);
		array_two_variable_constraint=NULL;
	}
  if (count_two_variable_constraint != NULL)
	{
		free(count_two_variable_constraint);
		count_two_variable_constraint=NULL;
	}
  if (solutions != NULL)
	{
		free(solutions);
		solutions=NULL;
	}
  if (domain != NULL)
	{
		free(domain);
		domain=NULL;
	}

  if (order1 != NULL)
	{
		free(order1);
		order1=NULL;
	}

  if (order2 != NULL)
	{
		free(order2);
		order2=NULL;
	}

  if (constraint_node != NULL)
	{
		free(constraint_node);
		constraint_node=NULL;
	}
  if (order_node != NULL)
	{
		free(order_node);
		order_node=NULL;
	}

  if (var_node_head != NULL)
	{
		free(var_node_head);
		var_node_head=NULL;
	}
  if (var_node_tail != NULL)
	{
		free(var_node_tail);
		var_node_tail=NULL;
	}

  if (dom_node_head != NULL)
	{
		free(dom_node_head);
		dom_node_head=NULL;
	}
  if (dom_node_tail != NULL)
	{
		free(dom_node_tail);
		dom_node_tail=NULL;
	}
  if (neighbor != NULL)
	{
		free(neighbor);
		neighbor=NULL;
	}
  if (newvalue != NULL)
	{
		free(newvalue);
		newvalue=NULL;
	}
  if(new_order_node != NULL)
	{
		free(new_order_node);
		new_order_node=NULL;
	}
  if (vector != NULL)
	{
		free(vector);
		vector=NULL;
	}




	count_constant=0;
	count_variable=0;
	number_binary_constraint=0;
	number_lub=0;
	number_order=0;

	number_constraint_ee=0;	/*number_constraint_ee: a count for the Constraint-ee */
	number_constraint_ev=0;	/*number_constraint_ev: a count for the Constraint-ev */
	number_constraint_ve=0;	/*number_constraint_ve: a count for the Constraint-ve */
	number_constraint_vv=0;	/*number_constraint_vv: a count for the Constraint-vv */
	count_lub_constraint=0;  	/*count_lub_constraint: a count for the upper-bound-constraints */

	count_lub_unit=2; 
	count_solution=0;	

}
	
