%----
% Copyright (C) 1996, Gregor Meyer, FernUniversitaet Hagen, Germany.
% This file is part of the system 'Typical'.
% See the file LICENSE for conditions of distribution and use.
%----
%:- multifile (subtype)/1, (suptype)/1, (type)/1, (pred)/1, (type_import)/1, (local)/1.
















%  csp_h.pl
%
% Interface between Prolog and C-modul 
% 1996 Georg Veltmann

:- ensure_loaded(typeops).

:- type_import(prolog_types).
:- type_import(prolog_meta).
:- type_import(prolog_num).
:- type_import(prolog_misc). 
 
% declaration of external C-funtions from csp.o

foreign_file('csp.o',[free_space,constant_num,constant,order_num,order_def_less,variable_num,variable,binary_constraint_num,constraint_def_ee,constraint_def_ev,constraint_def_ve,constraint_def_vv,upper_bound_constraint_num,ub_first_e,ub_first_v,ub_expr_e,ub_expr_v,constraint_def_eu,constraint_def_vu,check_unique,solvable_first,solvable_all,less_neighbor,set_null_elem,reset_constraints,test_leq,glb,lub,is_maximal,is_minimal,get_count_variable,get_count_solution,get_variables,get_solutions,get_newvalue,get_neighbor]).

% Each C-function has an assigned prolog-clause with the same name:

% freeing all the memory used by C-modul

:- pred c_free_space.
foreign(free_space,c_free_space).

% reset all constraints, but do not affect the domain build with constants and
% order relations

:- pred c_reset_constraints.
foreign(reset_constraints,c_reset_constraints).

% An element is defined as the NULL_Elem

:- pred c_set_null_elem(top).
foreign( set_null_elem,c_set_null_elem(+atom)).

% functions for defining the number of constants and the constants themselves 

:- pred c_constant_num(nat).
foreign(constant_num,c_constant_num(+integer)).

:- pred c_constant(top).
foreign(constant,c_constant(+atom)).

% functions for defining the number of order relations and the 
% relations themselves

:- pred c_order_num(nat).
foreign(order_num,c_order_num(+integer)).

:- pred c_order_def_less(top,top).
foreign(order_def_less,c_order_def_less(+atom,+atom)).
%	first argument =< second argument

% functions for defining the number of variables and the variables themselves

:- pred c_variable_num(nat).
foreign(variable_num,c_variable_num(+integer)).

:- pred c_variable(top).
foreign(variable,c_variable(+atom)).

% functions for defining binary constraints

:- pred c_binary_constraint_num(nat).
foreign(binary_constraint_num,c_binary_constraint_num(+integer)).
:- pred c_constraint_def_ee(top,top).
foreign(constraint_def_ee,c_constraint_def_ee(+atom,+atom)).  /* a =< b */
:- pred c_constraint_def_ev(top,top).
foreign(constraint_def_ev,c_constraint_def_ev(+atom,+atom)).  /* a =< v */
:- pred c_constraint_def_ve(top,top).
foreign(constraint_def_ve,c_constraint_def_ve(+atom,+atom)).  /* u =< b */
:- pred c_constraint_def_vv(top,top).
foreign(constraint_def_vv,c_constraint_def_vv(+atom,+atom)).  /* u =< v */

% functions for defining upper bound constraints

% number of upper bound constraints
:- pred c_upper_bound_constraint_num(nat).
foreign(upper_bound_constraint_num,c_upper_bound_constraint_num(+integer)).

% starting a set for which the upper bound is defined
:- pred c_ub_first_e(top,top).
foreign(ub_first_e,c_ub_first_e(+atom,[-address])).
:- pred c_ub_first_v(top,top).
foreign(ub_first_v,c_ub_first_v(+atom,[-address])).

% inserting more terms into the set
:- pred c_ub_expr_e(top,top,top).
foreign(ub_expr_e,c_ub_expr_e(+address, +atom,[-address])). /* return: u \up a */
:- pred c_ub_expr_v(top,top,top).
foreign(ub_expr_v,c_ub_expr_v(+address, +atom,[-address])). /* return: u \up v */

% closing the list and defining the term that is the upper bound
:- pred c_constraint_def_eu(top,top).
foreign(constraint_def_eu,c_constraint_def_eu(+atom, +address)).  /* a = u */
:- pred c_constraint_def_vu(top,top).
foreign(constraint_def_vu,c_constraint_def_vu(+atom, +address)).  /* v = u */

% checks if all least upper bounds in given order are unique and
% introduces new constants and order relations to make them unique

:- pred c_check_unique(int).
foreign(check_unique,c_check_unique([-integer])).

% start solving the defined CSP for first solution

:- pred c_solvable_first(int).
foreign(solvable_first,c_solvable_first([-integer])).
%	return:  1    constraints solvable 
%                0    constraints not solvable 

% get all other solutions

:- pred c_solvable_all(int).
foreign(solvable_all,c_solvable_all([-integer])).
%	return:  1    constraints solvable 
%                0    constraints not solvable 

% more Prolog-like solvable()/0 clause
:- pred solvable.
solvable :- 
	c_solvable_first(F),
	!, F = 1.

% search the direct neighbors under element 

:- pred c_less_neighbor(int).
foreign(less_neighbor,c_less_neighbor([-integer])).

% calculate the least-upper-bound of element a and b

:- pred c_lub(top,top,top).
foreign(lub,c_lub(+atom, +atom,[-atom])).

% calculate the greatest-lower-bound of element a and b

:- pred c_glb(top,top,top).
foreign(glb,c_glb(+atom, +atom,[-atom])).

% test element a =< element b

:- pred c_test_leq(top,top,int).
foreign(test_leq,c_test_leq(+atom, +atom,[-integer])).

% test that atom is maximal element in the given order relation
 
:- pred c_is_maximal(top,nat).
foreign(is_maximal,c_is_maximal(+atom,[-integer])).

% test that atom is minimal element in the given order relation 

:- pred c_is_minimal(top,nat).
foreign(is_minimal,c_is_minimal(+atom,[-integer])).

% clauses for access to global result variables of the C-module

:- pred c_get_count_variable(nat).
foreign(get_count_variable,c_get_count_variable([-integer])).
:- pred c_get_count_solution(nat).
foreign(get_count_solution,c_get_count_solution([-integer])).
:- pred c_get_variables(nat,top).
foreign(get_variables,c_get_variables(+integer,[-atom])).
:- pred c_get_solutions(nat,nat,top).
foreign(get_solutions,c_get_solutions(+integer,+integer,[-atom])).
:- pred c_get_newvalue(nat,top).
foreign(get_newvalue,c_get_newvalue(+integer,[-atom])).
:- pred c_get_neighbor(nat,top).
foreign(get_neighbor,c_get_neighbor(+integer,[-atom])).

% load the module as object file into Prolog

:- load_foreign_files(['csp.o'],[]).








































































