%----
% Copyright (C) 1996, Gregor Meyer, FernUniversitaet Hagen, Germany.
% This file is part of the system 'Typical'.
% See the file LICENSE for conditions of distribution and use.
%----
%:- multifile (subtype)/1, (suptype)/1, (type)/1, (pred)/1, (type_import)/1, (local)/1.
















%
% Implementation of predicates declared in lib.pl
%
% This variant is for plain Standard Prolog

:- type_import(lib).

%-------
is_list([]).
is_list([_|_]).
 
%-------
member(X,[X|_]).
member(X,[_|T]) :- member(X,T).
 
%---- 
memberchk(Element, [Element|_]) :- !.
memberchk(Element, [_|Rest]) :-
        memberchk(Element, Rest).

 
%---- 
non_member(_,[]).
non_member(X, [X1|L]) :-
        X \== X1, non_member(X, L).


last(X,L) :- append(_,[X],L).
 
%-------
same_length([],[]).
same_length([_|L1],[_|L2]) :- same_length(L1,L2).

%-------
append([],L,L).
append([H|T],L,[H|TL]) :- append(T,L,TL).

% deterministic append
det_append(A,B,AB) :- append(A,B,AB).

%------- 
reverse([],[]).
reverse([H|T],R) :- reverse(T,RT),append(RT,[H],R).

%-------
delete([], _, []).
delete([Elem1 | Rest_of_old], Elem1, New_list) :-
        !,delete(Rest_of_old, Elem1, New_list).
delete([Elem2 | Rest_of_old], Elem1, [Elem2|Rest_of_new]) :- 
        !,delete(Rest_of_old, Elem1, Rest_of_new).
 
%-------
remove_duplicates([],[]).
remove_duplicates([X|Xs],Ys) :-
        member(X,Xs), !,
        remove_duplicates(Xs,Ys).
remove_duplicates([X|Xs],[X|Ys]) :-
        % not member(X,Xs),
        remove_duplicates(Xs,Ys).

%-------
subset([],_).
subset([X|Xs],S) :- 
        member(X,S),
        subset(Xs,S).
 
%-------
subtract([], _, []).
subtract([Element|Residue], Set, Difference) :-
        memberchk(Element, Set), !,
        subtract(Residue, Set, Difference).
subtract([Element|Residue], Set, [Element|Difference]) :-
        subtract(Residue, Set, Difference).
 
%-------
variant(A, B) :-
        subsumes_chk(A,B),
        subsumes_chk(B,A).
 
