%----
% Copyright (C) 1996, Gregor Meyer, FernUniversitaet Hagen, Germany.
% This file is part of the system 'Typical'.
% See the file LICENSE for conditions of distribution and use.
%----
%:- multifile (subtype)/1, (suptype)/1, (type)/1, (pred)/1, (type_import)/1, (local)/1.

















:- ensure_loaded(typeops).
/*
% ----------------------------------------------------------------------
% Operators used in type declarations.
% same as in file 'typeops' !
:- op(1199, fx, [(local), (abstract)]).  % module keywords
:- op(1198, fx, [(subtype), (suptype), (type), (pred), (pred_neg), (predl), (rule)]).
:- op(1197, xfy, (-->)).
:- op(1196, xfy, (with)).
% do not redefine op (;)

% ----------------------------------------------------------------------
*/

%type program --> prog(typenames,type-order,clauses).
%type passone --> pass1(typenames,type-order,clauses,preddecl,constructors).

/* see config.pl
file_search_path(tap,'.').
file_search_path(tap,lib).
file_search_path(tap,tap_lib).
*/

:- dynamic pragma_option/1.

% command line options must appear before files
%
tap(['-v'|Args]) :-
	assert(pragma_option(verbose)),
	tap(Args).

tap(['-dummy-pdecl'|Args]) :-
	assert(pragma_option(dummy_pdecl)),
	tap(Args).

tap(['-dummy-fdecl'|Args]) :-
	assert(pragma_option(dummy_fdecl)),
	tap(Args).

tap(X) :- 
	% add built-in type goal, must be know for valid_type_names
	Pass1 = pass1([goal|_],_,_,_,_),
	get_from(X,Pass1), % Pass 1: Read Prog + Decls from file
	!,
	process_pass1(Pass1,Program), % Pass 2: process input with known type equivalents
        process_input(Program),	% Process the Program
	write(end),nl,
	halt.
	
get_from([],_Pass1) :- !.	

get_from([File|FileList],Pass1) :- !,
        get_from(File,Pass1),   
        get_from(FileList,Pass1).

get_from(File,Pass1) :-
        atom(File), !,
	write('tap: checking '),write(File),nl,
        read_input(File,all,Pass1).  % Read from the source file

get_from(File,_Pass1) :-
        write('tap>>  invalid file name '),
        write(File), 
	write(' .halt'), nl, halt.

:- dynamic io_opened_module/1.

% * Load file FileName, Mode is transformation-mode (all or nonlocal)
%   Files is a list of already read files
read_input(FileName0,Mode,Pass1) :- 
        % write(read_input(tap(FileName0))),flush_output(user),
	file_dealiased(tap(FileName0),FileName),
        % write('  dealiased':FileName),flush_output(user),
	% don't read a file twice
	absolute_file_name(FileName,AbsName),
        % write('  absolute':AbsName),nl,flush_output(user),
	(io_opened_module(AbsName) -> true      % already read
	; assert(io_opened_module(AbsName)),
	  (pragma_option(verbose) ->
	   (write(read_input(FileName,Mode)), nl);
            true),
	  read_input2(AbsName,Mode,Pass1)
	).

read_input2(FileName,Mode,Pass1) :- 
	open(FileName,read,Stream),
	read_prg(Stream,Mode,Pass1),
	close(Stream).

read_prg(Stream,Mode,Pass1) :-
	read(Stream,Cl),
	!,
	expand_term(Cl,Clause),   % handles things such as grammar rules
	!,
  	% write(got(Clause)),nl,
	( Clause == end_of_file -> true   % , write(eof)
	; transform(Clause,Mode,Pass1), !,
	  read_prg(Stream,Mode,Pass1)
	).
	
% ----------------------------------------------------------------------
process_pass1(pass1(Ts, Os, Cl, Pr, Co), Program) :-
	close_list(Ts), close_list(Os), close_list(Cl),close_list(Pr), close_list(Co),
	check_type_decls(Os, Co),
	process_types(Ts, Program),
	process_order(Os, Program),
	process_clauses(Cl, Program),
	process_pdecls(Pr, Program),
	process_constructors(Co, Program).

check_type_decls(Os, Co) :-
	check_dupl(Co),
	check_alias_order(Os),
	check_alias_constructor(Co).

check_dupl([]).
check_dupl([c(Type,_Constructor) | TL]) :-
	(member(c(Type, _X), TL) -> 
            (write('multiple definition of type '), write(Type), nl , halt) ;
	    check_dupl(TL)).

check_alias_order([]).
check_alias_order([X < Y | OL]) :-
	(type_equiv_map(X,_Type1) -> 
            (write('type alias '),write(X), 
	     write(' used in subtype relationship '), write(X < Y),
	     nl, halt);
	    (type_equiv_map(Y,_Type2) -> 
		(write('type alias '), write(Y), 
		 write(' used in subtype relationship '), write(X < Y),
		 nl, halt);
	        check_alias_order(OL))).

check_alias_constructor([]).
check_alias_constructor([c(Type, Constructor) | CL]) :-
	(type_equiv_map(Type,_Type1) -> 
            (write('type alias '), write(Type),
	     write(' also defined through '), write(Type),
	     write(' --> '), write(Constructor),
	     nl, halt);
	     check_alias_constructor(CL)).


process_types([], _).
process_types([T | TL], Program) :- 
	process_type(Program, T),
	process_types(TL, Program).

process_order([], _).
process_order([O | OL], Program) :- 
	process_subt(Program, O),
	process_order(OL, Program).

process_clauses(Cl,prog(_Ts, _Os, Cl)).

process_pdecls([], _).
process_pdecls([p(Pred, Predkind) | PL], Program) :- 
	process_pdecl(Program, Pred, Predkind),
	process_pdecls(PL, Program).
process_pdecls([pc(Pred, Predkind, Co) | PL], Program) :- 
	process_pdecl(Program, Pred, Predkind, Co),
	process_pdecls(PL, Program).

process_constructors([], _).
process_constructors([c(Type, Constructor) | CL], Program) :- 
	func_transform(Type,Constructor,Program),
	process_constructors(CL, Program).


% ----------------------------------------------------------------------
process_input(Program) :- !,
	Program = prog(Ts,Os,Cls),
	close_list(Ts), close_list(Os), close_list(Cls),
        % add 'xx', often chosen as arbitrary instantiation in var-Typing
	po_create([xx|Ts],Os,PO),
	true,
	true,
	((func_decl_variant_ngr0(X1,Xn,X2,X3),
	    true,fail) ; true),
	((pred_decl_variant_ngr0(X1,Xn,_Xp,X2,X3),
	    true,fail) ; true),
	true,
	type_infer_cl(PO,Cls).
/*
	Lat = PO,
	(option_with_bottom -> complete_bottom(Lat,TO1); TO1=Lat),
	(pragma_option(with_top) -> 
    	    complete_top(TO1,TO2)
	;   TO2 = TO1
	),
	type_infer_cl(TO2,Cls).
*/

% ----------------------------------------------------------------------
:- dynamic type_equiv_map/2.
% type_equiv_map(Name,Canon) :-
% Name is or is an abbreviation for Canon.

transform(_,_Mode, _Pass1) :- true, fail.

transform((:- op(N,A,Op)),_, _Pass1) :- ! , 
	% define operator, otherwise read can't handle new Op's
	op(N,A,Op).

transform((:- ensure_loaded(_)), _, _Pass1) :- !.

transform((local _), nonlocal, _Pass1) :- !.
transform((local Any), all, Pass1) :- !,
	transform(Any, all, Pass1).

transform((abstract (type Name --> _) ), nonlocal, Pass1) :- !,
	transform((type Name),nonlocal,Pass1).

transform((abstract (type Name == _)), nonlocal, Pass1) :- !,
	transform((type Name),nonlocal,Pass1).
transform((abstract (type Name = _)), nonlocal, Pass1) :- !,
	transform((type Name),nonlocal,Pass1).

transform((abstract (type TDef)), all, Pass1) :- !,
	transform((type TDef), all, Pass1).

transform((type pragma(Option)), all, _Pass1) :- !,
	assert(pragma_option(Option)).

transform(type_import(FileName), _, Pass1) :- !,
	read_input(FileName,nonlocal,Pass1).
transform(type_impex(FileName), _, Pass1) :- !,  % import and export all
	read_input(FileName,nonlocal,Pass1).


% Mycroft OKeefe type-checker syntax.
% 	do not take queries (:-goal) as facts (goal)
%	head condition (genericity) differs
transform((:- type_import(FileName)), Mode, Pass1) :- !,
	transform(type_import(FileName) , Mode,Pass1).
transform((:- type_impex(FileName)), Mode, Pass1) :- !,
	transform(type_impex(FileName) , Mode,Pass1).
transform((:- abstract Definition), Mode, Pass1) :- !,
	transform((abstract Definition) , Mode,Pass1).
transform((:- local Definition), Mode, Pass1) :- !,
	transform((local Definition) , Mode,Pass1).
transform((:- type Definition), Mode, Pass1) :- !,
	transform((type Definition) , Mode,Pass1).
transform((:- subtype Definition), Mode, Pass1) :- !,
	transform((subtype Definition) , Mode,Pass1).
transform((:- suptype Definition), Mode, Pass1) :- !,
	transform((suptype Definition) , Mode,Pass1).
transform((:- pred Declaration), Mode, Pass1) :- !,
	transform((pred Declaration) , Mode,Pass1).
transform((:- predl Declaration), Mode, Pass1) :- !,
	transform((predl Declaration) , Mode,Pass1).
transform((:- pred_neg Declaration), Mode, Pass1) :- !,
	transform((pred_neg Declaration) , Mode,Pass1).
transform((:- rule Declaration), Mode, Pass1) :- !,
	transform((rule Declaration) , Mode,Pass1).

% n. Subtype relationship
transform((subtype Type1 < Type2) ,_, Pass1) :- !,
	store_subt(Pass1, Type1 < Type2).
transform((type Type1 < Type2) ,_, Pass1) :- !,
	store_subt(Pass1, Type1 < Type2).
transform((suptype Type1 > Type2) ,_, Pass1) :- !,
	store_subt(Pass1, Type2 < Type1).
transform((type Type1 > Type2) ,_, Pass1) :- !,
	store_subt(Pass1, Type2 < Type1).

%  Type Constructor declarations and Function type declarations
transform((type Type --> Constructors) ,_, Pass1) :- !,
	store_type(Pass1,Type),
	store_constructor(Pass1,Type, Constructors).

transform((type Type == RealType) ,_,  Pass1) :- !,
	store_type_equiv(Pass1,Type,RealType).
transform((type Type = RealType) ,_,  Pass1) :- !,
	store_type_equiv(Pass1,Type,RealType).
%  Just Type constructors : no function symbols
transform((type Type) ,_, Pass1) :- !,
	store_type(Pass1,Type).

% 3. Predicate type declaration

/*
transform((pred P1 ; P2), _, Pass1) :- !,
	transform((pred P1), _, Pass1),
	transform((pred P2), _, Pass1).
*/

transform((pred {Pred}), _, Pass1) :- !,
	store_pdecl(Pass1, Pred,pred).
transform((predl {Pred}), _, Pass1) :- !,
	store_pdecl(Pass1, Pred,predl).
transform((pred Pred -> Co), Mode, Pass1) :- !,  % style variant
	transform((pred Pred with Co), Mode,Pass1).
transform((predl Pred -> Co), Mode, Pass1) :- !,  % style variant
	transform((predl Pred with Co), Mode,Pass1).
transform((pred Pred with Co), _, Pass1) :- !,
	store_pdecl(Pass1, Pred,pred,Co).
transform((predl Pred with Co), _, Pass1) :- !,
	store_pdecl(Pass1, Pred,predl,Co).
transform((pred_neg Pred | Co), _, Pass1) :- !,
	store_pdecl(Pass1, Pred,pred_neg,Co).
transform((pred_neg Pred with Co), _, Pass1) :- !,
	store_pdecl(Pass1, Pred,pred_neg,Co).
transform((pred Pred), _, Pass1) :- !,
	store_pdecl(Pass1, Pred,pred).
transform((predl Pred), _, Pass1) :- !,
	store_pdecl(Pass1, Pred,predl).
transform((pred_neg Pred), _, Pass1) :- !,
	store_pdecl(Pass1, Pred,pred_neg).
transform((rule NonTerminal), _, Pass1) :- !, % grammar rule
	NonTerminal =.. [F|Args],
	append(Args,[list(top),list(top)],A2),
	Pred =.. [F|A2],
	store_pdecl(Pass1, Pred,predl). 


% End of file: 
transform(end_of_file, _, _Pass1) :- !,
	write('eof should be handled elsewhere').


% Else it is a program clause
transform(Cl, all, Pass1) :- 
	Pass1 = pass1(_T,_O,Cls,_Pr,_Co), 
	insert_end(Cl, Cls), !.

transform(_Cl, nonlocal, _Pass1) :- !.

transform(Cl,_,_) :- write('can not transform '), write(Cl),nl,halt.

% insert in open list.
insert_end(X, L) :- var(L), !, L = [X|_NewL].
insert_end(X, [_|L]) :- insert_end(X, L).

% close open list.
close_list([]) :- !.
close_list([_|L]) :- close_list(L).

store_type_equiv(_P, Type, RealType) :- 
	check_type_name(Type),
	assert(type_equiv_map(Type, RealType)).

store_type(pass1(Ts,_Os,_Cl,_Pr,_Co), Type) :-
	insert_end(Type, Ts).

store_subt(pass1(_Ts,Os,_Cl,_Pr,_Co), X < Y) :-
	insert_end( X < Y, Os).

store_pdecl(pass1(_Ts,_Os,_Cl,Pr,_Co), Pred, Predkind) :-
	insert_end( p(Pred, Predkind), Pr).
store_pdecl(pass1(_Ts,_Os,_Cl,Pr,_Co), Pred, Predkind, Co) :-
	insert_end( pc(Pred, Predkind, Co), Pr).

store_constructor(pass1(_Ts,_Os,_Cl,_Pr,Co), Type, Constructor) :- 
	insert_end( c(Type, Constructor), Co).

% ----------------------------------------------------------------------
process_pdecl(prog(S,_,_), Pred,PredKind) :-
	Pred =.. [PName|Args],
	functor(Pred,_,Arity),
	\+ test_pred_decl_variant_ngr(PName,Arity,_,_),  % must be unique
	!,
	trans_pdecl_list(Args,Args2,Co),
	normalize_type_equiv_list(Args2,Args3),
	valid_type_names_list(S,Args3),
	normalize_type_equiv_list(Co,Co4),
	valid_type_names_co(S,Co4),
	simplify_pdecl(Args3,Co4,Co5),
	de_store_pdecl(PName,Arity,PredKind,Args3,Co5).
process_pdecl(_,Pred,_) :- 
	write(' multiple declaration for predicate '), write(Pred),nl,halt.

process_pdecl(prog(S,_,_), Pred,PredKind,Co0) :-
	Pred =.. [PName|Args],
	functor(Pred,_,Arity),
	\+ test_pred_decl_variant_ngr(PName,Arity,_,_),  % must be unique
	!,
	commas_to_l(Co0,Co),
	trans_pdecl_list(Args,Args2,Co2),
	append(Co,Co2,Co3),
	normalize_type_equiv_list(Args2,Args3),
	valid_type_names_list(S,Args3),
	normalize_type_equiv_list(Co3,Co4),
	valid_type_names_co(S,Co4),
	simplify_pdecl(Args3,Co4,Co5),
	de_store_pdecl(PName,Arity,PredKind,Args3,Co5).
process_pdecl(_,Pred,_,_) :-
	write(' multiple declaration for predicate '), write(Pred),nl,halt.

commas_to_l((A,B),[A|BL]) :- !, commas_to_l(B,BL).
commas_to_l(A,[A]).

check_type_name(bottom) :- 
	% type bottom is forbidden
	write('type bottom is forbidden, use another typename.'),
	!,fail.    
check_type_name(Type) :-
	% parameters must be pairwise distinct variables.
	Type =.. [_F|Args],
	pairwise_distinct_var(Args),
	!.
check_type_name(Type) :-
	write(Type:'parameters must be pairwise distinct variables.'),
	fail.

pairwise_distinct_var([]).
pairwise_distinct_var([X|Xs]) :-
	var(X),
	\+ var_member(X,Xs),
	pairwise_distinct_var(Xs).
	

process_type(prog(Ts,_Os,_Cl), Type) :- 
	check_type_name(Type),
	functor(Type,TC,N),
	( N=0 -> 
	    TSymb = TC
	;   TSymb = TC   % store arity somewhere ?
	),
	insert_uniq_open(TSymb,Ts) , !.
process_type(_,Type) :- write('can not store type '), write(Type),nl,halt.


process_subt(prog(_Ts,Os,_Cl), X < Y) :-
	normalize_type_equiv(X,Xe),
	normalize_type_equiv(Y,Ye),
	Xe =..[FX|ArgsX],
	Ye =..[FY|ArgsY],
	ArgsX == ArgsY,    % parameters must be in same sequence 
	!,
	insert_uniq_open(FX =< FY,Os).       % only =< in poset !
process_subt(_, X < Y) :-
	write('subtyping not allowed '), write(X<Y),
	fail.

store_fdecl(prog(S,_,_), Type, Functor,Arity, ArgList) :- 
	normalize_type_equiv_list(ArgList,Args3),
	valid_type_names_list(S,Args3),
	de_store_fdecl(Functor,Arity,Args3,Type).

store_fdecl_goal(_Program,Type,Var,Goal) :-
	%write(fgoal:(func_decl_variant_ngr0(Var,_,[],Type) :- Goal) ),nl,
	de_store_fdecl_goal(Type,Var,Goal).
	
% ----------------------------------------------------------------------

% Process the function symbols on the Rhs of  type Lhs --> Rhs
% Insert their types in Fd.

func_transform(Type, (Cons1;Cons2),Program) :- !,
	f_transform(Type, Cons1, Program), 
	func_transform(Type,Cons2,Program).

func_transform(Type, Cons1,Program) :- f_transform(Type,Cons1,Program).

% Process a function symbol 
% ----------------------------------------------------------------------
f_transform(Type,Term,Program) :- 
	Term = {Var|Goal},
	store_fdecl_goal(Program,Type,Var,Goal).

f_transform(Type,Term,Program) :- 
	Term = {Var^Goal},
	store_fdecl_goal(Program,Type,Var,Goal).

f_transform(Type,Term,Program) :- 
	Term =.. [Functor | ArgList],
	% type_preserving(Functor,ArgList,Type),  not required !?
	!,
	functor(Term,_,Arity),
	store_fdecl(Program,Type,Functor,Arity,ArgList).

f_transform(_Type,Term,_Program) :- 
	Term =.. [Functor | _ArgList],
	write('function symbol '), write(Functor), 
	write(' is not type preserving; Cannot continue '), nl, halt.



normalize_type_equiv(Term,T9) :- 
	var(Term), !,
	T9 = Term.
normalize_type_equiv(Term,T9) :-
	% \+ var(Term),
	type_equiv_map(Term,T2), !,
	normalize_type_equiv(T2,T9).
normalize_type_equiv(Term,T9) :-
	% \+ var(Term),
	% \+ type_equiv_map(Term,T2), !,
	Term =.. [F|Args],
	normalize_type_equiv_list(Args,Args2),
	T9 =.. [F|Args2].

normalize_type_equiv_list([],[]).
normalize_type_equiv_list([X|Xs],[Y|Ys]) :-
	normalize_type_equiv(X,Y),
	normalize_type_equiv_list(Xs,Ys).

%----
% valid_type_names(Symbols,TypeTerm) :-
%	every functor in TypeTerm is member of Symbols.
%
valid_type_names(_Ss,Term) :-
        var(Term), !.
valid_type_names(_Ss,top) :- !.
valid_type_names(Ss,Term) :-
        % \+ var(Term),
	Term =.. [F|Args],
	memberchk_open(F,Ss), 
	!,
        valid_type_names_list(Ss,Args).
valid_type_names(Ss,Term) :-
	nl,
	write('type symbol unknown '), write(Term), nl,
	write('not in ':Ss),nl,
	halt.
 
valid_type_names_list(_Ss,[]).
valid_type_names_list(Ss,[X|Xs]) :-
        valid_type_names(Ss,X),
        valid_type_names_list(Ss,Xs).

valid_type_names_co(_Ss,[]). 
valid_type_names_co(Ss,[X=<Y|C]) :-
	valid_type_names(Ss,X),
	valid_type_names(Ss,Y),
	valid_type_names_co(Ss,C).

type_preserving(Func,Domain,Target) :-
	numbervars(Target,0,_),
	\+ ground(Domain),
	!,
	write((Func:Domain -> Target) ), write(' not type preserving'),
	fail.
type_preserving(_,_,_).



