%----
% Copyright (C) 1996, Gregor Meyer, FernUniversitaet Hagen, Germany.
% This file is part of the system 'Typical'.
% See the file LICENSE for conditions of distribution and use.
%----
%:- multifile (subtype)/1, (suptype)/1, (type)/1, (pred)/1, (type_import)/1, (local)/1.

















% :- module tc_andor.
%
% build type-checking sequences from logic formulas
%
%

% :- ensure_loaded(library(lists)).  % append, member.

%-------  test-calls
% tc_tree_pl1(p,X),tc_sequence(X,Y).
% tc_tree_pl1(and(p,q),X),tc_sequence(X,Y).
% tc_tree_pl1(and(p,and(q,r)),X),tc_sequence(X,Y).
% tc_tree_pl1(and(and(p,q),r),X),tc_sequence(X,Y).
% tc_tree_pl1(and(p,or(q,r)),X),tc_sequence(X,Y).
% tc_tree_pl1(or(p,or(q,r)),X),tc_sequence(X,Y).
% tc_tree_pl1(not(q),X),tc_sequence(X,Y).
% tc_tree_pl1(and(not(q),not(r)),X),tc_sequence(X,Y).
% tc_tree_pl1(and(p,and(not(q),not(r))),X),tc_sequence(X,Y).
% aos1(impl(or(q,r),p), X, Y).
% aos1(impl(not(female),male), X, Y).
%
% aos2( (p:-q,r), X, Y).
% aos2( (p:-q,\+ r), X, Y).
% aos2( (p:- \+(r),q,\+(s)), X, Y).
% aos2( (p;s:-q), X, Y).
% aos2( (p:-q;r), X, Y).
% aos2( (\+(male):-female), X, Y).
% aos2( (male,\+(female)), X, Y).
% aos2( (p:-male,\+(female)), X, Y).
% aos2( (p:-(q1,q2);(r1,r2)), X, Y).
% aos2( (p:-((q1,q2);(r1,r2)),s), X, Y).

pred aos1(top,top).
pred aos1(top,top,top).
pred aos2(top,top).
pred aos2(top,top,top).

aos1(F,Y) :- aos1(F,_,Y).
aos1(F,X,Y) :- tc_tree_pl1(F,X),tc_sequence(X,Y).
aos2(F,Y) :- aos2(F,_,Y).
aos2(F,X,Y) :- tc_tree_prolog(F,X),tc_sequence(X,Y).
%-------test


% type ao --> ao(list(ao),list(ao)) ;   % ao(Ands,Ors)
%             leaf(any).


% aolists(AO:ao,Ands:list(ao),Orslist(ao)).
%
aolists(leaf(X),[leaf(X)],[]).
aolists(ao(A,O),A,O).


% tc_tree_pl1(Formula,AO-Tree) :-
%	Formula is a formula in first order predicate logic.
%	AO-Tree is an And/Or-Tree representing the
%	structure of Formula w.r.t. type-consistency.
%
% pred tc_tree_pl1( +formula, -ao).
%
tc_tree_pl1(forall(_Vars,F),AO) :- !,
	tc_tree_pl1(F,AO).
tc_tree_pl1(exist(_Vars,F),AO) :- !,
	tc_tree_pl1(F,AO).
tc_tree_pl1(and([X]),AO) :- !,
	tc_tree_pl1(X,AO).
tc_tree_pl1(and([X,Y]),AO) :- !,
	tc_tree_pl1(X,AOx),
	tc_tree_pl1(Y,AOy),
	aolists(AOx,Ax,Ox),
	aolists(AOy,Ay,Oy),
	append(Ax,Ay,A),
	append(Ox,Oy,O),
	AO = ao(A,O).
tc_tree_pl1(and([A1|As]),AO) :-	!,
	tc_tree_pl1(and([A1,and(As)]),AO).
tc_tree_pl1(or([X]),AO) :- !,
	tc_tree_pl1(X,AO).
tc_tree_pl1(or([X,Y]),AO) :- !,
	tc_tree_pl1(X,Ox),
	tc_tree_pl1(Y,Oy),
	AO = ao([],[Ox,Oy]).
tc_tree_pl1(or([A1|As]),AO) :- !,
	tc_tree_pl1(or([A1,or(As)]),AO).
tc_tree_pl1(not(X),AO) :- !,
	tc_tree_pl1(X,Ox),
	AO = ao([],[Ox]).
tc_tree_pl1(implies(X,Y),AO) :- !,    % X => Y
	ao(or([not(X),Y]),AO).  % !?
tc_tree_pl1(implied(X,Y),AO) :- !,    % X <= Y
	ao(or([X,not(Y)]),AO).  % !?
tc_tree_pl1(equiv(X,Y),AO) :- !,    % X <=> Y
	tc_tree_pl1(and([X,Y]),AO).  % !?

tc_tree_pl1(X,leaf(X)).

% tc_tree_prolog(Formula,AO-Tree) :-
%	see above. Formula is a Prolog clause.
%
tc_tree_prolog(':-'(X,Y),AO) :- !,    % Prolog   X :- Y.
	tc_tree_prolog(','(X,Y),AO).
tc_tree_prolog(','(X,Y),AO) :- !,
	tc_tree_prolog(X,AOx),
	tc_tree_prolog(Y,AOy),
	aolists(AOx,Ax,Ox),
	aolists(AOy,Ay,Oy),
	append(Ax,Ay,A),
	append(Ox,Oy,O),
	AO = ao(A,O).
tc_tree_prolog(';'(X,Y),AO) :- !,
	tc_tree_prolog(X,Ox),
	tc_tree_prolog(Y,Oy),
	AO = ao([],[Ox,Oy]).
tc_tree_prolog(\+ X,AO) :- !,
	tc_tree_prolog(X,Ox),
	AO = ao([],[Ox]).
tc_tree_prolog(X,leaf(X)).


% tc_sequence(AO-Tree,Seq) :-
%	AO-Tree is an And/Or-Tree.
%	Seq is a sequence of leafs, that have to be checked for 
%	type-consistence simultaneously.
%	tc_sequence/2 may produce several sequences via backtracking.
%	All sequences will be checked separately later.
%
% pred tc_sequence( +ao, -list(atom)).
%
tc_sequence(leaf(X),[X]).
tc_sequence(ao([],[]),[]).
tc_sequence(ao([X|L],O),LS) :-
	tc_sequence(X,L1),
	tc_sequence(ao(L,O),L2),
	append(L1,L2,LS).
tc_sequence(ao([],L),LS) :-
	member(X,L),
	tc_sequence(X,LS).

