%----
% Copyright (C) 1996, Gregor Meyer, FernUniversitaet Hagen, Germany.
% This file is part of the system 'Typical'.
% See the file LICENSE for conditions of distribution and use.
%----
%:- multifile (subtype)/1, (suptype)/1, (type)/1, (pred)/1, (type_import)/1, (local)/1.
















:- ensure_loaded(typeops).

:- type_import(ti_2po).


type typeterm = top.       %
 
type tt_pum == pum(typeterm).  % principal upper matcher problem over typeterms
 
type tt_co1 = po_constraint(typeterm).   % constraint over type terms
 
 
type ttx_pum ==    % principal upper matcher over typeterms, exploded
        pum(top).
 
type ttx_co = list(po_constraint(top)).
 
% imploded  exploded:
% int   ->   int
% list(int) -> [list,int]
% list(list(int)) -> [list,[list,int]]
 
 

% for checks
pred simple_pums(list(ttx_pum)).
simple_pums([]).
simple_pums([pum(_,L)|PUMs]) :-
        simple_pum_constraints(L),
        simple_pums(PUMs).


pred simple_pum_constraints(list(pum_constr(_))).
simple_pum_constraints(_L) :- true, fail.
simple_pum_constraints([]).
simple_pum_constraints([X|Xs]) :-
        simple_pum_co(X),
        simple_pum_constraints(Xs).

pred simple_pum_co(pum_constr(_)).
simple_pum_co(_X) :- true, fail.
simple_pum_co(X=<=Y) :- simple(X),simple(Y).



/*

% type terms over alphabet Sym
type tterm(Sym) -->
	tt(Sym,list(tterm(Sym))).

% int -> tt(int,[])
% list(int) -> tt(list,[tt(int,[])])

% explode constraint over type-terms in Prolog-repr.
% into c. over type-terms as tt(...)


pred explode_Ctt(list(top),list(top)).
%
explode_Ctt([],[]).
explode_Ctt([XopY|C],[XUopYU|C2]) :-
        XopY =.. [Op,X,Y],  check_top( member(Op,[*,:,<,=<,=<=,=,lub])),
        univrec_tt(X,XU),
        univrec_tt(Y,YU),
        XUopYU =..[Op,XU,YU],
        explode_Ctt(C,C2).

pred implode_Ctt(list(top),list(top)).
%
implode_Ctt([],[]).
implode_Ctt([XUopYU|C],[XopY|C2]) :-
        XUopYU =.. [Op,XU,YU],  check_top( member(Op,[*,:,<,=<,=<=,=,lub])),
        univrecb_tt(XU,X),
        univrecb_tt(YU,Y),
        XopY =..[Op,X,Y],
        implode_Ctt(C,C2).

*/

pred explode_simpum(list(tt_pum),list(ttx_pum)).
%
explode_simpum([],[]).
explode_simpum([P|PUMs],[Pe|PUMse]) :-
	explode_pum(P,Pe),
	explode_simpum(PUMs,PUMse).


pred explode_pum(tt_pum,ttx_pum).
%
explode_pum(pum(Pid,L),pum(Pid,Le)) :-
	explode_C(L,Le).


pred explode_C(list(top),list(top)).

explode_C([],[]).
explode_C([lub(X,L)|C],[XULU|C2]) :-
	!,
        univrec(X,XU),
        univrec_list(L,LU),
	XULU = lub(XU,LU),
        explode_C(C,C2).
explode_C([XopY|C],[XUopYU|C2]) :-
        XopY =.. [Op,X,Y],  true,
        univrec(X,XU),
        univrec(Y,YU),
        XUopYU =..[Op,XU,YU],
        explode_C(C,C2).

pred implode_C(list(top),list(top)).
 
implode_C([],[]).
implode_C([lub(XU,LU)|C],[XL|C2]) :-
	!,
        univrecb(XU,X),
        univrecb_list(LU,L),
        XL = lub(X,L),
        implode_C(C,C2).
implode_C([XUopYU|C],[XopY|C2]) :-
        XUopYU =.. [Op,XU,YU],  true,
        univrecb(XU,X),
        univrecb(YU,Y),
        XopY =..[Op,X,Y],
        implode_C(C,C2).

 
pred univrec(top,top).
%
univrec(X,X) :- simple(X), !. % (var(X); X =.. [_FX])
univrec(L,LU) :- L=[_|_], !, univrec_list(L,LU).
% Element ist evtl. schon vorher einmal zerlegt worden.
univrec(X,L) :-
        X =.. [FX|ArgsX],
        univrec_list(ArgsX,ArgsXL),
        L = [FX|ArgsXL].

pred univrec_list(list(top),list(top)).
univrec_list([],[]).
univrec_list([X|Xs],[XU|XUs]) :-
        univrec(X,XU),
        univrec_list(Xs,XUs).


% inverse
pred univrecb(top,top).

univrecb(X,X) :- simple(X), !. % (var(X); X =.. [_FX])
univrecb(L,X) :-
        L = [FX|ArgsXL], var(FX), !,
        univrecb_list(ArgsXL,ArgsX),
        X = [FX|ArgsX].
univrecb(L,X) :-
        L = [FX|ArgsXL], \+ var(FX), !,
        univrecb_list(ArgsXL,ArgsX),
        X =.. [FX|ArgsX].
univrecb(Term,Term).
 
pred univrecb_list(list(top),list(top)).
univrecb_list([],[]).
univrecb_list([X|Xs],[XU|XUs]) :-
        univrecb(X,XU),
        univrecb_list(Xs,XUs).
 

pred greatest_lower_match(poset(_),top,top).
greatest_lower_match(_L, X,Y) :- true, var(X), !, X=Y.
greatest_lower_match(L, X,Y) :-
        \+ var(X), \+ var(Y),
        X =.. [XFunc|XArgs],    % variables in XArgs assumed to be distinct
        Y =.. [YFunc|YArgs],
        po_test_leq(L, XFunc,YFunc),
        greatest_lower_matches(L, XArgs,YArgs).
        
pred greatest_lower_matches(poset(_),list(top),list(top)).
greatest_lower_matches(_L, [],[]).
greatest_lower_matches(L, [X|Xs],[Y|Ys]) :-
        greatest_lower_match(L, X,Y),
        greatest_lower_matches(L, Xs,Ys).
 


% from metutl.pl (R. O'Keefe)
%
pred variables_of(top,list(top)). 
%
variables_of(Term, Vars) :-
        variables_of(Term, [], Vars).

pred variables_of(top,list(top),list(top)). 
%
variables_of(Term, Sofar, Sofar) :-
        var(Term),
        var_member_chk(Term, Sofar),
        !.
variables_of(Term, Sofar, [Term|Sofar]) :-
        var(Term),
        !.
variables_of(Term, Sofar, Vars) :-
        functor(Term, _, N),
        variables_of(N, Term, Sofar, Vars).

 
pred variables_of(nat,top,list(top),list(top)). 
%
variables_of(0, _, Vars, Vars) :- !.
variables_of(N, Term, Sofar, Vars) :-
        arg(N, Term, Arg),
        variables_of(Arg, Sofar, Mid),
        M is N-1, !,
        variables_of(M, Term, Mid, Vars).
 
pred var_member_chk(top,list(top)). 
%
var_member_chk(Var, [Head|_]) :-
        Head == Var,
        !.
var_member_chk(Var, [_|Tail]) :-
        var_member_chk(Var, Tail).



